/**
 * The XMOJO Project 5
 * Copyright  2003 XMOJO.org. All rights reserved.

 * NO WARRANTY

 * BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
 * THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
 * OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
 * PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
 * OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
 * TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
 * LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
 * REPAIR OR CORRECTION.

 * IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
 * ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
 * THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
 * GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
 * USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
 * DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
 * PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
 * EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGES.
**/

package com.adventnet.adaptors.html;

import java.io.File;
import java.util.Collection;
import java.util.Iterator;
import java.util.Properties;
import java.util.Vector;
import java.net.URL;

import com.adventnet.agent.logging.Log;
import com.adventnet.agent.logging.LogFactory;
import com.adventnet.agent.utilities.xml.XMLNode;
import com.adventnet.agent.utilities.xml.XMLDataReader;
import com.adventnet.agent.utilities.xml.XMLDataWriter;

import org.mortbay.util.*;
import org.mortbay.jetty.*;
import org.mortbay.http.*;

/**
 * This class plugs in the Jetty WebServer to the HtmlAdaptor.
 *
 * @see HttpServerInterface
 */
public class JettyHtmlServer implements HttpServerInterface
{
	private static final int defaultPortNumber = 8030;

    private String configFileName ="conf/http/etc/JettyConfig.xml";
    private String jarConfig = "conf/http/etc/JarConfig.xml";

    private Properties props = null;
    private boolean authentication = false;
    private int port = defaultPortNumber;
    private boolean isStarted = false;
    private boolean sslSupport = false;
    private boolean isJarFile = false;
    private boolean fromSetPort = false;

    private Server server = null;

    private Log log = null;

    /**
     * Primary constructor for the HttpServerImpl without any configuration options.
     */
    public JettyHtmlServer()
    {
		this(defaultPortNumber, false, false, false);
    }

    /**
     * Secondary constructor with configuration options.
     *
     * Note: For this version, authentication is disabled and hence will not
	 * have any effect by setting authentication to true.
	 *
     * @param port The port to start the WebServer
     *
     * @param authentication The boolean flag to turn ON/OFF
	 * 				Authentication of the Requests. Default is false.
     */
    public JettyHtmlServer(int port, boolean authentication)
    {
        this(port, authentication, false, false);
    }

    /**
     * Constructor with configuration options.
     *
     * Note: For this version, authentication and SSL support are disabled and
	 * hence will not have any effect by setting true to those parameters.
	 *
     * @param port The port to start the WebServer
     *
     * @param authentication The boolean flag to turn ON/OFF
	 * 				Authentication of the Requests. Default is false.
     *
     * @param sslSupport The boolean flag to turn ON/OFF SSL Support.
	 * 				Default is false.
     */
    public JettyHtmlServer(int port, boolean authentication, boolean sslSupport)
	{
		this(port, authentication, sslSupport, false);
    }

    /**
     * Constructor with configuration options.
     *
     * Note: For this version, authentication and SSL support are disabled and
	 * hence will not have any effect by setting true to those parameters.
	 *
     * @param port The port to start the WebServer
     *
     * @param authentication The boolean flag to turn ON/OFF
	 * 				Authentication of the Requests. Default is false.
     *
     * @param sslSupport The boolean flag to turn ON/OFF SSL Support.
	 * 				Default is false.
     *
     * @param isJarFile Boolean flag to indicate configuration files taken from Jar.
     */
    public JettyHtmlServer(int port, boolean authentication,
                           boolean sslSupport, boolean isJarFile)
    {
		log = getLogObject();
        this.authentication = authentication;
        this.sslSupport = sslSupport;
        this.port = port;
        this.isJarFile = isJarFile;

		if(!isJarFile)
		{
            XMLDataReader xmld =
                new XMLDataReader(HtmlAdaptor.getParentDirectory() + File.separator
                                  + configFileName );
            XMLNode rootNode = xmld.getRootNode();

            Vector vec = rootNode.getChildNodes();
            changePort(rootNode, (new Integer(port)).toString());

            changeSSL(rootNode, sslSupport);
            changeAgentDir(rootNode, HtmlAdaptor.getParentDirectory());

            XMLDataWriter xw = new XMLDataWriter(HtmlAdaptor.getParentDirectory() + File.separator + configFileName ,rootNode);
        }
    }

    /**
     * Setter for the configuration file Name with location.
     *
     * @param configFileName The configuration file name with path.
     */
    public void setConfigFileName(String configFileName)
    {
        this.configFileName = configFileName;

        try
		{
            restartHttpServer();
        }
		catch (Exception ex)
		{
            log.error("Exception", ex);
        }
    }

    /**
     * Getter for the configFileName.
     *
     * @return The configuration file Name by which the Server is running.
     */
    public String getConfigFileName()
    {
        return this.configFileName;
    }

    /**
     * Setter for the configuration file Name which will be inside the
     * jar containing this class.
     *
     * @param jarConfig The configuration file name with path as in the jar File.
     */
    public void setConfigJarFile(String jarConfig)
    {
        this.jarConfig = jarConfig;

        try
		{
            restartHttpServer();
        }
		catch(Exception ex)
		{
            log.error("Exception", ex);
        }
    }

    /**
     * Setter for the configuration file Name which will be inside the
     * jar containing this class.
     *
     * @return  The configuration file name by which the Server is running
     * 				with path as in the jar File.
     */
    public String getConfigJarFile()
    {
        return this.jarConfig;
    }

    /**
     * Returns the port number in which the http server gets started.
     *
     * @return the port number.
     *
     * @see setPort(Integer port)
     */
    public Integer getPort()
    {
        return new Integer(port);
    }

    /**
     * Sets the port number in which the http server to get started.
     *
     * @param the port number of the http web-server.
     *
     * @see getPort()
     */
    public void setPort(Integer port) throws Exception
    {
	    setPort(port.intValue());
    }

    /**
     * Sets the port number in which the http server to get started.
     *
     * @param the port number of the http web-server.
     *
     * @see getPort()
     */
    public void setPort(int port)
    {
        if(port < 0 || port > 65535)
		{
            return;
        }

		if(port != this.port)
		{
            this.port = port;

            if(!isJarFile)
			{
				XMLDataReader xmld =
                    new XMLDataReader(HtmlAdaptor.getParentDirectory()
										+ File.separator + configFileName );

				XMLNode rootNode = xmld.getRootNode();

				changePort(rootNode, String.valueOf(port));

				XMLDataWriter xw =
                    new XMLDataWriter(HtmlAdaptor.getParentDirectory()
								+ File.separator + configFileName, rootNode);
            }
			else
			{
                fromSetPort = true;
            }

			//restart the Http server at the specified port
			log.info("applying the new port number by restarting the Http server");

			try
			{
            	restartHttpServer();
				log.info("successfully changed to new port number");
			}
            catch(Exception ex)
            {
            	log.error("Exception while trying to change server port number", ex);
				log.fatal("Failed to change to new port number");
			}
        }
    }

    /**
     * Setter for enabling and disabling Authentication of the WebServer.
     *
     * Note: For this version, this is disabled and hence will not have
	 * any effect by calling this method.
     *
     * @param auth The flag for turning ON/OFF the authentication of WebServer.
     */
    public void setAuthentication(boolean auth)
    {
	    //this.authentication = auth;
        this.authentication = false;
    }

    /**
     * To know whether the authentication is enabled in the WebServer.
     *
     * Note: For this version, this is disabled and hence will always return false.
	 *
     * @return true if the authentication of the WebServer is turned ON else false.
     */
    public boolean isAuthentication()
    {
        return this.authentication;
    }

    /**
     * This method adds a new User with the userName and password
     * to the Authentication of the WebServer.
     *
     * Note: For this version, this is disabled and hence will not have
	 * any effect by calling this method.
     *
     * @param userName The user name to be added.
     *
     * @param password The password for the user.
     */
    public void addUser(String userName, String passwd)
    {
	    //this is not supported in this release
    }

    /**
     * Setter for enabling and disabling SSL Support of the WebServer.
     *
     * Note: For this version, this is disabled and hence will not have
	 * any effect by calling this method.
	 *
     * @param sslSupport The flag for turning ON/OFF the SSL Support of WebServer.
     */
    public void setSslSupport(boolean sslSupport)
    {
        //this.sslSupport = sslSupport;
        this.sslSupport = false;
    }

    /**
     * To know whether the SSL Support is enabled in the WebServer.
     *
     * Note: For this version, this is disabled and hence will always return false.
     *
     * @return true if the SSL Support of the WebServer is turned ON else false.
     */
    public boolean isSslSupport()
    {
        return this.sslSupport;
    }

    /**
     * Starts the HttpServer - Jetty WebServer.
     *
     * @see stopHttpServer()
     *
     * @exception Exception on error while starting WebServer.
     */
    public void startHttpServer() throws Exception
    {
    	System.setProperty("LOG_FILE", "logs/agent.log");
		System.setProperty("LOG_CLASSES", "org.mortbay.util.WriterLogSink");

        if(!isJarFile)
		{
            server = new Server(HtmlAdaptor.getParentDirectory()+
                                File.separator+configFileName);
        }
        else
		{
            URL url = this.getClass().getResource("/"+jarConfig);
            server = new Server(url);
            if((sslSupport) || (fromSetPort))
			{
                //removes the listener configured through xml file.
                Collection lisCol =  server.getListeners();
                if(!(lisCol.isEmpty())){
                    Iterator iter = lisCol.iterator();
                    if(iter.hasNext())
                        server.removeListener((HttpListener)iter.next());
                }

                if(!sslSupport)
				{
                    SocketListener socLis = new SocketListener(new InetAddrPort(this.port));
                    server.addListener(socLis);
                }
            }
        }

        server.start();
        isStarted = true;
    }

    /**
     * Stops the WebServer.
     *
     * @exception Exception on error while stopping WebServer.
     */
    public void stopHttpServer() throws Exception
    {
        server.stop();
        isStarted = false;
    }

    /**
     * Restarts the WebServer.
     *
     * @exception Exception while error trying to restart the WebServer
     */
    public void restartHttpServer() throws Exception
    {
        try
		{
            stopHttpServer();
        }
		catch(Exception ee)
		{
            throw new Exception("Exception while stopping WebServer in restart");
        }

		try
		{
            startHttpServer();
        }
		catch(Exception eee)
		{
            throw new Exception("Exception while starting WebServer in restart");
        }
    }

    void changeAgentDir(XMLNode node, String agentDir)
    {
        if (node.getNodeName().equals("Call")
            && node.getAttributeList().get("name").equals("addContext"))
		{
            XMLNode setNode = (XMLNode)node.getChildNodes().elementAt(1);

            if (setNode.getAttributeList().get("name").equals("ResourceBase"))
			{
                XMLNode sysNode = (XMLNode)setNode.getChildNodes().elementAt(0);
                sysNode.getAttributeList().put("default", agentDir+"/conf/http/ssi");
                return;
            }

        }

        Vector childVec = node.getChildNodes();
        if(childVec != null)
		{
            int len = childVec.size();

            for(int i = 0; i <len; i++)
			{
                changeAgentDir((XMLNode)childVec.elementAt(i), agentDir);
            }
        }
    }

    private boolean changeSSL(XMLNode node, boolean flag)
    {
        if (node.getNodeName().equals("Call")
            && node.getAttributeList().get("name").equals("addListener"))
		{
            XMLNode argNode = (XMLNode)node.getChildNodes().elementAt(0);
            XMLNode newNode = (XMLNode)argNode.getChildNodes().elementAt(0);

            if(flag)
			{
                newNode.getAttributeList().put("class", "org.mortbay.http.SunJsseListener");

                XMLNode setNode = null;

                Vector vec= newNode.getChildNodes();
                int size = vec.size();
                boolean keyStore = true;
                boolean keyPassword = true;
                boolean password = true;

                String keyStoreStr = null;
                String keyPasswordStr = null;
                String passwordStr = null;

                XMLDataReader myReader =
                    new XMLDataReader(HtmlAdaptor.getParentDirectory() + File.separator
                                      + "etc" + File.separator + "HtmlAdaptorSSLConf.xml");
                XMLNode myConfigNode = myReader.getRootNode();
                Vector configChilds = myConfigNode.getChildNodes();
                int configSize = configChilds.size();
                for (int j = 0; j < configSize; j++)
				{
                    XMLNode configChild = (XMLNode)configChilds.get(j);
                    if (configChild.getNodeName().equals("Store") &&
                        configChild.getAttribute("name").equals("keyStore"))
					{
                        Vector properties = configChild.getChildNodes();
                        int propsSize = properties.size();
                        for (int k = 0; k < propsSize; k++)
						{
                            XMLNode prop = (XMLNode)properties.get(k);
                            String value = (String)prop.getAttribute("name");
                            if (value.equals("Name"))
							{
                                keyStoreStr = (String)prop.getAttribute("value");
                            }
                            else if (value.equals("Password"))
							{
                                passwordStr = (String)prop.getAttribute("value");
                            }
                            else if (value.equals("Keypassword"))
							{
                                keyPasswordStr = (String)prop.getAttribute("value");
                            }

                            if (keyPasswordStr != null &&
                                passwordStr != null &&
                                keyStoreStr != null)
							{
                                break;
                            }
                        }
                    }
                }

                Vector newNodeChilds = newNode.getChildNodes();
                int newNodeChildSize = newNodeChilds.size();
                for (int i = 0; i < newNodeChildSize; i++)
				{
                    XMLNode newNodeChild = (XMLNode)newNodeChilds.get(i);
                    if (newNodeChild.getNodeName().equals("Set"))
					{
                        if (newNodeChild.getAttribute("name").equals("Keystore"))
						{
                            newNodeChild.removeFromParent();
                        }
                        else if (newNodeChild.getAttribute("name").equals("Password"))
						{
                            newNodeChild.removeFromParent();
                        }
                        else if (newNodeChild.getAttribute("name").equals("KeyPassword"))
						{
                            newNodeChild.removeFromParent();
                        }
                        else if (newNodeChild.getAttribute("name").equals("ConfigFileName"))
						{
						    newNodeChild.removeFromParent();
                        }
                    }
                }

                XMLNode addNode = new XMLNode();
                addNode.setNodeType(XMLNode.ELEMENT);
                addNode.setNodeName("Set");

                addNode.setAttribute("name", "Keystore");
                addNode.setNodeValue("etc/adventnetkey");
                newNode.addChildNode(addNode);

                addNode = new XMLNode();
                addNode.setNodeType(XMLNode.ELEMENT);
                addNode.setNodeName("Set");

                addNode.setAttribute("name", "KeyPassword");
                addNode.setNodeValue("adventnet");
                newNode.addChildNode(addNode);

                addNode = new XMLNode();
                addNode.setNodeType(XMLNode.ELEMENT);
                addNode.setNodeName("Set");

                addNode.setAttribute("name", "Password");
                addNode.setNodeValue("adventnet");
                if (password)
				{
                    newNode.addChildNode(addNode);
                }
                return true;
            }
            else
			{
                if (newNode.getAttributeList().get("class").equals("org.mortbay.http.SunJsseListener"))
				{
                    newNode.getAttributeList().put("class", "org.mortbay.http.SocketListener");

                    XMLNode setNode = null;
                    for (int i=0; i<newNode.getChildNodes().size(); i++)
					{
                        setNode = (XMLNode)newNode.getChildNodes().elementAt(i);

                        if (setNode.getAttributeList().get("name").equals("KeyPassword") ||
                            setNode.getAttributeList().get("name").equals("Keystore") ||
                            setNode.getAttributeList().get("name").equals("Password"))
						{
                            setNode.removeFromParent();
                            // return true;
                        }
                    }
                }
            }
        }

        Vector childVec = node.getChildNodes();

        if(childVec != null)
		{
            int len = childVec.size();
            for (int i = 0; i <len; i++)
			{
                if (changeSSL((XMLNode)childVec.elementAt(i), flag))
				{
                    return true;
                }
            }
        }

        return false;
    }

    //this method changes the value of port in the config file
    private void changePort(XMLNode node, String port)
    {
        if (node.getNodeName().equals("Call")
            && node.getAttributeList().get("name").equals("addListener"))
		{
            XMLNode argNode = (XMLNode)node.getChildNodes().elementAt(0);
            XMLNode newNode = (XMLNode)argNode.getChildNodes().elementAt(0);

            XMLNode setNode = null;

            for (int i=0; i<newNode.getChildNodes().size(); i++)
			{
                setNode = (XMLNode)newNode.getChildNodes().elementAt(i);

                if (setNode.getAttributeList().get("name").equals("Port"))
				{
                    setNode.setNodeValue(port);
                    return;
                }
            }
        }

        Vector childVec = node.getChildNodes();
        if(childVec != null)
		{
            int len = childVec.size();

            for (int i = 0; i <len; i++)
			{
                changePort((XMLNode)childVec.elementAt(i),port);
            }
        }
    }

    private Log getLogObject()
    {
        try
		{
            log = LogFactory.getInstance("HTML");
        }
        catch(InstantiationException ie)
		{
            // ie.printStackTrace();
        }

        return log;
    }
}