/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package com.adventnet.jmx;

import java.io.File;
import java.io.FileInputStream;
import java.util.Hashtable;

import com.adventnet.agent.logging.Log;
import com.adventnet.agent.logging.LogFactory;

/**
 * A simple test class loader capable of loading from multiple sources,
 * such as local files or a URL. Must be subclassed and the abstract method
 * loadClassBytes() implemented to provide the preferred source.
 *
 */
public class DefaultClassLoader extends ClassLoader
{
	//---------- Fields --------------------------------------
	private Hashtable classes = new Hashtable();
	private char      classNameReplacementChar;

	protected boolean   monitorOn = false;
	protected boolean   sourceMonitorOn = true;

	private String filePrefix = File.separator;
	private Log log = null;

	//---------- Initialization ------------------------------
	public DefaultClassLoader()
	{
		try
		{
			log = LogFactory.getInstance("JMX");
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	//---------- Public Methods ------------------------------

	//---------- Superclass Overrides ------------------------

	/**
	 * This is a simple version for external clients since they
	 * will always want the class resolved before it is returned
	 * to them.
	 */
	public Class loadClass(String className) throws ClassNotFoundException
	{
	    return (loadClass(className, true));
	}

	//---------- Abstract Implementation ---------------------
	public synchronized Class loadClass(String className, boolean resolveIt)
											throws ClassNotFoundException
	{
	    Class   result;
	    byte[]  classBytes;

	    //----- Check our local cache of classes
	    result = (Class)classes.get(className);
	    if (result != null)
		{
	        return result;
	    }

	    //----- Check with the primordial class loader
	    try
		{
	        result = super.findSystemClass(className);

	        return result;
	    }
		catch (ClassNotFoundException e)
		{
	        log.debug(">> Not a system class.");
	    }

	    //----- Try to load it from preferred source
	    // Note loadClassBytes() is an abstract method
	    classBytes = loadClassBytes(className);

	    if (classBytes == null)
		{
	        throw new ClassNotFoundException();
	    }

	    //----- Define it (parse the class file)
	    result = defineClass(null, classBytes, 0, classBytes.length);
	    if (result == null)
		{
	        throw new ClassFormatError();
	    }

	    //----- Resolve if necessary
	    if (resolveIt) resolveClass(result);

	    // Done
	    classes.put(className, result);
	    log.debug(">> Returning newly loaded class.");

	    return result;
	}

	/**
	 * This optional call allows a class name such as "COM.test.Hello" to be
	 * changed to "COM_test_Hello", which is useful for storing classes from
	 * different packages in the same retrival directory.
	 * In the above example the char would be '_'.
	 */
	public void setClassNameReplacementChar(char replacement)
	{
	    classNameReplacementChar = replacement;
	}

	private byte[] loadClassBytes(String className)
	{
		className = formatClassName(className);

	    byte result[];
	    String fileName = filePrefix + className;
		//String fileName = className;

	    try
		{
	        FileInputStream inStream = new FileInputStream(fileName);

	        result = new byte[inStream.available()];
	        inStream.read(result);
	        inStream.close();

	        return result;
	    }
		catch (Exception e)
		{
	        // If we caught an exception, either the class
	        // wasn't found or it was unreadable by our process.
			log.debug("loadClassBytes",e);
	        return null;
	    }
	}

	protected String formatClassName(String className)
	{
	    if (classNameReplacementChar == '\u0000')
		{
	        // '/' is used to map the package to the path
	        return className.replace('.', '/') + ".class";
	    }
		else
		{
	        // Replace '.' with custom char, such as '_'
	        return className.replace('.',
	            classNameReplacementChar) + ".class";
	    }
	}
} // End class