/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management;

import java.io.Serializable;
import java.io.ObjectInputStream;
import java.io.ObjectInputStream.GetField;
import java.io.ObjectStreamField;
import java.io.ObjectOutputStream;
import java.io.ObjectOutputStream.PutField;
import java.io.IOException;
import java.util.Vector;

/**
 * This class implements of the {@link javax.management.NotificationFilter NotificationFilter}
 * interface for the {@link javax.management.AttributeChangeNotification attribute change notification}.
 * The filtering is performed on the name of the observed attribute.
 * <P>
 * It manages a list of enabled attribute names.
 * A method allows users to enable/disable as many attribute names as required.
 */
public class AttributeChangeNotificationFilter
					implements NotificationFilter, Serializable
{
    /**
     * Vector that contains the enabled attribute names.
	 * The default value is an empty vector.
     */
	private Vector enabledAttributes = new Vector();

	/* Serial version UID */
	private static final long serialVersionUID = 0xa7e9cc49419e9f53L;

	private static final ObjectStreamField[] serialPersistentFields =
	{
		new ObjectStreamField("enabledAttributes", java.util.Vector.class)
	};

	/**
	 * The noArg constructor .
	 */
	public AttributeChangeNotificationFilter()
	{
		//enabledAttributes = new Vector();
	}

    /**
     * Invoked before sending the specified notification to the listener.
     * <BR>This filter compares the attribute name of the specified attribute
	 * change notification with each enabled attribute name. If the attribute
	 * name equals one of the enabled attribute names, the notification
	 * must be sent to the listener and this method returns <CODE>true</CODE>.
     *
     * @param notification The attribute change notification to be sent.
     *
     * @return <CODE>true</CODE> if the notification has to be sent to
	 * 				the listener, <CODE>false</CODE> otherwise.
     */
	public boolean isNotificationEnabled(Notification notification)
	{
		if(notification == null)
			return false;

		if( !(notification instanceof AttributeChangeNotification) )
			return false;

		String name = ((AttributeChangeNotification)notification).getAttributeName();

		for(int i=0;i<enabledAttributes.size();i++)
		{
			String str = (String)enabledAttributes.elementAt(i);
			if(name.equals(str))
				return true;
		}

		return false;
	}


	/**
	 * Disables all the attribute names.
	 */
	public void disableAllAttributes()
	{
		enabledAttributes.removeAllElements();
	}

    /**
     * Disables all the attribute change notifications the attribute name of
	 * which equals the specified attribute name to be sent to the listener.
	 * <BR>If the specified name is not in the list of enabled attribute names,
	 * this method has no effect.
     *
     * @param name The attribute name.
     */
	public void disableAttribute(java.lang.String name)
	{
		if(name == null)
			return;

		for(int i=0;i<enabledAttributes.size();i++)
		{
			String str = (String)enabledAttributes.elementAt(i);
			if(name.equals(str))
				enabledAttributes.removeElementAt(i);
		}
	}

    /**
     * Enables all the attribute change notifications the attribute name of
	 * which equals the specified name to be sent to the listener.
     * <BR>If the specified name is already in the list of enabled attribute
	 * names, this method has no effect.
     *
     * @param name The attribute name.
     *
     * @exception java.lang.IllegalArgumentException The attribute
	 * 				name parameter is null.
     */
	public void enableAttribute(java.lang.String name)
		throws java.lang.IllegalArgumentException
	{
		if(name == null)
			throw new IllegalArgumentException(name);

		boolean flag = false;
		for(int i=0;i<enabledAttributes.size();i++)
		{
			String str = (String)enabledAttributes.elementAt(i);
			if(name.equals(str))
				flag = true;
		}

		if(!flag)
			enabledAttributes.add(name);
	}

    /**
     * Gets all the enabled attribute names for this filter.
     *
     * @return The list containing all the enabled attribute names.
     */
	public Vector getEnabledAttributes()
	{
		return enabledAttributes;
	}

	//-------------------------- Private methods ---------------------------//

	private void readObject(ObjectInputStream objectinputstream)
			        			throws IOException, ClassNotFoundException
	{
		ObjectInputStream.GetField getfield = objectinputstream.readFields();

		try
		{
			enabledAttributes=(Vector)getfield.get("enabledAttributes", null);
		}
		catch(Exception exception)
		{
		}
	}

	private void writeObject(ObjectOutputStream objectoutputstream)
								throws IOException
	{
		ObjectOutputStream.PutField putfield = objectoutputstream.putFields();
		putfield.put("enabledAttributes", enabledAttributes);
		objectoutputstream.writeFields();
	}
}