/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management;

import java.io.Serializable;

/**
 * This interface represents the behavioral metadata set for a JMX Element.
 * For examples, a descriptor is part of the ModelMBeanInfo,
 * ModelMBeanNotificationInfo, ModelMBeanAttributeInfo,
 * ModelMBeanConstructorInfo, and ModelMBeanParameterInfo.
 * <p>
 * A descriptor consists of a collection of fields. Each field is in
 * fieldname=fieldvalue format.
 * <p>
 * All field names and values are not predefined. New fields can be defined
 * and added by any program. In the case of ModelMBean some fields have been
 * predefined for consistency of implementation and support by the
 * ModelMBeanInfo ModelMBean*Info, and ModelMBean classes.
 */
public interface Descriptor extends Serializable
{
	/**
	 * Returns the value for a specific fieldname.
	 *
	 * @param fieldName The field name in question; if not found, null is returned.
	 *
	 * @return Object Field value.
	 *
	 * @exception RuntimeOperationsException for illegal value for field name.
	 *
	 */
	public Object getFieldValue(String fieldName)
						throws RuntimeOperationsException;

	/**
	 * Sets the value for a specific fieldname.	The field value will be
	 * validated before it is set.  If it is not valid, then an exception
	 * will be thrown. This will modify an existing field or add a new field.
	 *
	 * @param String fieldName: The field name to be set. Cannot be null or empty.
	 *
	 * @param Object fieldValue: The field value to be set for the field name.
	 * 				Can be null.
	 *
	 * @exception RuntimeOperationsException for illegal value for field name
	 * 				or field value.
	 */
	public void setField(String fieldName, Object fieldValue)
						throws RuntimeOperationsException;

	/**
	 * Returns all of the  fields contained in this descriptor as a string array.
	 *
	 * @return String array of fields in the format <i>fieldName=fieldValue</i>.
	 * 				If the value of a field is not a String, then the toString()
	 * 				method will be called on it and the returned value used as
	 * 				the value for the field in the returned array. Object values
	 * 				which are not Strings will be enclosed in parens. If the
	 * 				descriptor is empty, you will get an empty array.
	 */
	public String[] getFields();

	/**
	 * Returns all the fields names in the descriptor.
	 *
	 * @return String array of fields names. If the descriptor is empty,
	 * 				you will get an empty array.
	 */
	public String[] getFieldNames();

	/**
	 * Returns all the field values in the descriptor as an array of
	 * Objects. The retuned values are in the same order as the fieldNames
	 * String array parameter.
	 *
	 * @param fieldNames String array of the names of the fields that the
	 * 				values should be returned for.  If the array is empty
	 * 				then an empty array will be returned.  If the array is
	 * 				'null' then all values will be returned.  If a field
	 * 				name in the array does not exist, then null is returned
	 * 				for the matching array element being returned.
	 *
	 * @return Object array of field values. If the descriptor is empty,
	 * 				you will get an empty array.
	 */
	public Object[] getFieldValues(String[] fieldNames);

	/**
	 * Removes a field from the descriptor.
	 *
	 * @param fieldName String name of the field to be removed. If the field
	 * 				is not found no exception is thrown.
	 */
	public void removeField(String fieldName);

	/**
	 * Sets all Fields in the list to the new value in with the same index in
	 * the fieldValue array.  Array sizes must match. The field value will be
	 * validated before it is set. If it is not valid, then an exception will
	 * be thrown. If the arrays are empty, then no change will take effect.
	 *
	 * @param fieldNames String array of field names. The array and array
	 * 				elements cannot be null.
	 *
	 * @param fieldValues Object array of the corresponding field values.
	 * 				The array cannot be null. Elements of the array can be null.
	 *
	 * @exception RuntimeOperationsException for illegal value for field Names
	 * 				or field Values. Neither can be null.  The array lengths
	 * 				must be equal. If the descriptor construction fails for
	 * 				any reason, this exception will be thrown.
	 */
	public void setFields(String[] fieldNames, Object[] fieldValues)
						throws RuntimeOperationsException;

	/**
	 * Returns a new Descriptor which is a duplicate of the Descriptor.
	 *
	 * @return This returns a duplicate of the Descriptor.
	 *
	 * @exception RuntimeOperationsException for illegal value for field Names
	 * 				or field Values. If the descriptor construction fails for
	 * 				any reason, this exception will be thrown.
	 */
	public Object clone() throws RuntimeOperationsException;

	/**
	 * Returns true if fieldValues are checked to be sure they are legal for
	 * the fieldNames.
	 *
	 * @return It returns true if the fieldValues checked are legal otherwise false
	 *
	 * @exception RuntimeOperationsException If the validity checking fails
	 * 				for any reason, this exception will be thrown.
	 */
	public boolean isValid() throws RuntimeOperationsException;
}