/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management;

import java.lang.reflect.Method;
import java.io.Serializable;
import java.io.ObjectInputStream;
import java.io.ObjectInputStream.GetField;
import java.io.ObjectOutputStream;
import java.io.ObjectOutputStream.PutField;
import java.io.ObjectStreamField;
import java.io.IOException;

/**
 * Describes an MBean attribute exposed for management.
 */
public class MBeanAttributeInfo extends MBeanFeatureInfo
									implements Serializable, Cloneable
{
    /**
     * The actual attribute type.
     */
    private String type = null;

    /**
     * The attribute read right.
     */
	private boolean isReadable = true;

    /**
     * Indicates if this method is a "is"
     */
    private boolean isIs = false;

    /**
     * The attribute write right.
     */
	private boolean isWritable = true;

    /**
     * The attribute read method.
     */
    private transient Method getter = null;

    /**
     * The attribute write method.
     */
    private transient Method setter = null;

    /* Serial version UID */
    private static final long serialVersionUID = 7043855487133450673L;

    private static final ObjectStreamField[] serialPersistentFields =
			{
				new ObjectStreamField("attributeType", java.lang.String.class),
				new ObjectStreamField("isWrite",java.lang.Boolean.TYPE),
				new ObjectStreamField("isRead",java.lang.Boolean.TYPE),
				new ObjectStreamField("is",java.lang.Boolean.TYPE),
			};

	/**
     * Constructs an <CODE>MBeanAttributeInfo</CODE> object.
     *
     * @param name The name of the attribute.
     * @param type The type or class name of the attribute.
     * @param description A human readable description of the attribute.
     * @param isReadable True if the attribute has a getter method, false otherwise.
     * @param isWritable True if the attribute has a setter method, false otherwise.
     * @param is True if this attribute has an "is" getter, false otherwise.
	 */
	public MBeanAttributeInfo(String name, String type, String description,
							   boolean isReadable, boolean isWritable, boolean isIs)
	{
		super(name, description);
		this.type = type;
		this.isReadable = isReadable;
		this.isWritable = isWritable;
		this.isIs = isIs;
	}

	/**
     * This constructor takes the name of a simple attribute, and Method
	 * objects for reading and writing the attribute.
     *
     * @param name The programmatic name of the attribute.
     *
     * @param description A human readable description of the attribute.
     *
     * @param getter The method used for reading the attribute value.
     * 				May be null if the property is write-only.
     *
     * @param setter The method used for writing the attribute value.
     *          	May be null if the attribute is read-only.
     *
     * @exception IntrospectionException There is a consistency problem in
	 * 				the definition of this attribute.
	 */
	public MBeanAttributeInfo(String name, String description,
							   Method getter, Method setter)
							throws IntrospectionException
	{
		super(name, description);

		if(getter == null)
			isReadable = false;
		else if(getter.getName().startsWith("is"))
			isIs = true;

		if(setter == null)
			isWritable = false;

		this.getter = getter;
		this.setter = setter;

		getAttributeType();
	}

	private void getAttributeType() throws IntrospectionException
	{
		Class type = null;
		String gettername = null;
		String settername = null;
		String newgettername = null;
		String newsettername = null;

		if(getter != null)
		{
			if(getter.getParameterTypes().length != 0)
			{
				throw new IntrospectionException("Invalid getter"+getter.getName());
			}
			type = getter.getReturnType();
			if(type == Void.TYPE)
			{
				throw new IntrospectionException("Invalid getter"+getter.getName());
			}

			gettername = getter.getName();

			if(isIs)
				newgettername = gettername.substring(2);
			else
				newgettername = gettername.substring(3);
		}

		if(setter != null)
		{
			Class params[] = setter.getParameterTypes();

			if(params.length != 1)
			{
				throw new IntrospectionException("Invalid setter"+setter.getName());
			}
			if(type != null && type != params[0])
			{
				throw new IntrospectionException("Invalid setter"+setter.getName());
			}

			type = params[0];
			if(name!=null)
			{
				settername = setter.getName();
				newsettername = settername.substring(3);

				if(getter != null)
				{
					if(!newgettername.equals(newsettername))
					{
						throw new IntrospectionException("Difference in getter and setter name");
					}
				}
			}
		}

		if(type != null)
		{
			this.type = type.getName();
		}
	}

	/**
	 * Returns the class name of the attribute.
	 *
	 * @return The class name is returned as String
	 */
	public String getType()
	{
		return type;
	}

	/**
	 * Whether the value of the attribute can be read.
	 *
	 * @return True if the attribute can be read, false otherwise.
	 */

	public boolean isReadable()
	{
		return isReadable;
	}

	/**
	 * Whether new values can be written to the attribute.
	 *
	 * @return True if the attribute can be written, false otherwise.
	 */

	public boolean isWritable()
	{
		return isWritable;
	}

	/**
	 * Indicates if this attribute has an "is" getter
	 * @return True if the attribute has an "is" getter otherwise false
	 */
	public boolean isIs()
	{
		return isIs;
	}

	/**
	 * Creates and returns a copy of this object.
	 *
	 * @return Creates a duplicate copy of the object
	 */
	public Object clone()
	{
		return new MBeanAttributeInfo(getName(), type, getDescription(),
									  isReadable, isWritable, isIs);
	}

	/**
	 * Returns a human readable version of the MBeanAttributeInfo instance.
	 *
	 * @return MBeanAttributeInfo is returned as a String
	 */
	public String toString()
	{
		return (super.toString() +
				"\n Type = "+ type +
				"\n IsReadable = "+ new Boolean(isReadable).toString() +
				"\n IsWritable = "+ new Boolean(isWritable).toString() +
				"\n IsIs = "+ new Boolean(isIs).toString() );
	}

	//------------------------- Private methods ---------------------------//

	private void readObject(ObjectInputStream objectinputstream)
		        throws IOException, ClassNotFoundException
	{
	    ObjectInputStream.GetField getfield = objectinputstream.readFields();

	    try
	    {
	        type=(String)getfield.get("attributeType", "");
	        isWritable=getfield.get("isWrite",false);
	        isReadable=getfield.get("isRead",false);
	        isIs=getfield.get("is",false);
	    }
	    catch(Exception exception)
	    {
	    }
	}

      private void writeObject(ObjectOutputStream objectoutputstream)
	          throws IOException
      {
          ObjectOutputStream.PutField putfield = objectoutputstream.putFields();
          putfield.put("attributeType", type);
          putfield.put("isWrite", isWritable);
          putfield.put("isRead", isReadable);
          putfield.put("is", isIs);
          objectoutputstream.writeFields();
      }
}