/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management;
 
import java.io.Serializable;
import java.io.ObjectInputStream;
import java.io.IOException;

/**
 * Allows to discover the management interface exposed by an MBean; that is, the
 * set of attributes and operations which are available for management operations.
 */
public class MBeanInfo implements Serializable, Cloneable
{
    /**
     * The MBean qualified name.
     */
	private String className = null;

    /**
     * The human readable description of the class.
     */
	private String description = null;

    /**
     * The MBean attribute descriptors.
     */
    private MBeanAttributeInfo[] attributes = null;

     /**
      * The MBean constructor descriptors.
      */
	private MBeanConstructorInfo[] constructors = null;

    /**
     * The MBean operation descriptors.
     */
	private MBeanOperationInfo[] operations = null;

    /**
     * The MBean notification descriptors.
     */
	private MBeanNotificationInfo[] notifications = null;

	/* Serial version UID */
 	private static final long serialVersionUID = 0xa6795e2f085689c9L;

	/**
	 * Default constructor for constructing an MBeanInfo.
	 */
	protected MBeanInfo()
	{
	}

	/**
     * Constructs an <CODE>MBeanInfo</CODE>.
     *
	 * @param name The name of the Java class of the MBean described
	 * 				by this MBeanInfo.
	 *
     * @param description A human readable description of the MBean (optional).
     *
     * @param attributes The list of exposed attributes of the MBean.
     *
     * @param constructors The list of public constructors of the MBean.
     *
     * @param operations The list of operations of the MBean.
     *
     * @param notifications The list of notifications emitted by an MBean.
	 */
	public MBeanInfo(String className,
					 String description,
					 MBeanAttributeInfo[] attributes ,
					 MBeanConstructorInfo[] constructors ,
					 MBeanOperationInfo[] operations,
					 MBeanNotificationInfo[] notifications)
	{
		this.className = className;
		this.description = description;
		this.attributes = attributes;
		this.constructors = constructors;
		this.operations = operations;
		this.notifications = notifications;
	}

	/**
	 * Creates and returns a copy of this object.
	 * @return A duplicate copy of this object is created
	 */
	public Object clone()
	{
		return new MBeanInfo(className,
					 description,
					 attributes ,
					 constructors ,
					 operations,
					 notifications);
	}

	/**
	 * Returns the MBean Java class name described by this MBeanInfo.
	 *
	 * @return the name of the Java class of the MBean described by this MBeanInfo.
	 */
	public String getClassName()
	{
		return className;
	}

	/**
	 * Returns a human readable description of the MBean.
	 *
	 * @return a human readable description of the MBean.
	 */
	public String getDescription()
	{
		return description;
	}

	/**
     * Returns the list of  attributes exposed for management.
     * Each attribute is described by an <CODE>MBeanAttributeInfo</CODE> object.
     *
     * @return  An array of <CODE>MBeanAttributeInfo</CODE> objects.
	 */
	public MBeanAttributeInfo[] getAttributes()
	{
		if(attributes!=null){
			return attributes;
		}else{
			return new MBeanAttributeInfo[0];
		}
	}

	/**
     * Returns the list of the public constructors  of the MBean. Each
	 * constructor is described by an <CODE>MBeanConstructorInfo</CODE> object.
     *
     * @return  An array of <CODE>MBeanConstructorInfo</CODE> objects.
	 */
	public MBeanConstructorInfo[] getConstructors()
	{
		if(constructors != null)
		{
			return constructors;
		}
		else
		{
			return new MBeanConstructorInfo[0];
		}
	}

	/**
     * Returns the list of operations  of the MBean.
     * Each operation is described by an <CODE>MBeanOperationInfo</CODE> object.
     *
     * @return  An array of <CODE>MBeanOperationInfo</CODE> objects.
	 */
	public MBeanOperationInfo[] getOperations()
	{
		if(operations!=null){
			return operations;
		}else{
		 	return new MBeanOperationInfo[0];
		}
	}

	/**
     * Returns the list of the notifications emitted by the MBean. Each
     * notification is described by an <CODE>MBeanNotificationInfo</CODE> object.
     *
     * @return  An array of <CODE>MBeanNotificationInfo</CODE> objects.
	 */
	public MBeanNotificationInfo[] getNotifications()
	{
		if(notifications!=null){
			return notifications;
		}else{
			return new MBeanNotificationInfo[0];
		}
	}

	//------------------------- Private methods ---------------------------//

	private void readObject(ObjectInputStream objectInputStream)
							throws IOException, ClassNotFoundException
	{
		try
		{
			objectInputStream.defaultReadObject();
		}
		catch(Exception e){}
	}
}
