/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management;

import java.lang.reflect.Method;
import java.io.Serializable;
import java.io.ObjectInputStream;
import java.io.ObjectInputStream.GetField;
import java.io.ObjectOutputStream;
import java.io.ObjectOutputStream.PutField;
import java.io.ObjectStreamField;
import java.io.IOException;

/**
 * Describes a management operation exposed by an MBean.
 */
public class MBeanOperationInfo extends MBeanFeatureInfo
								implements Serializable, Cloneable
{
	/**
	 * Indicates that the operation is a write-like in nature, and would
	 * modify the MBean in some way, typically by writing some value or
	 * changing a configuration.
	 */
	public static final int ACTION = 1;

	/**
	* Indicates that the operation is a read/write-like in nature.
	**/
	public static final int ACTION_INFO = 2;
	/**
	* Indicates that the operation is a read-like in nature, it basically returns some information.
	**/
	public static final int INFO = 3;
	/**
	* Indicates that the operation has an "unknown" nature.
	**/
	public static final int UNKNOWN = 4;

	//checkout the default values...

	private transient java.lang.reflect.Method method = null;
	private MBeanParameterInfo[] signature = null;
	private String type = "void";
	private int impact = INFO;

	/* Serial version UID */
	private static final long serialVersionUID = 0xaa40472295fe839eL;

	/**
     * Constructs an <CODE>MBeanOperationInfo</CODE> object.
     *
     * @param description A human readable description of the operation.
     *
     * @param method The <CODE>java.lang.reflect.Method</CODE> object
	 * 				describing the MBean operation.
	 */
	public MBeanOperationInfo (String description, Method method)
	{
		super(method.getName(), description);

		this.method = method;
		this.name = method.getName();
		this.type = method.getReturnType().getName();

		Class [] parameterTypes = method.getParameterTypes();

		if(parameterTypes.length == 0)
			this.signature = new MBeanParameterInfo[0];
		else
		{
			this.signature = new MBeanParameterInfo[parameterTypes.length];
			for(int i=0; i< parameterTypes.length; i++)
			{
				String paramName = "param"+i;
				String paramDesc = "";
				signature[i] = new MBeanParameterInfo(paramName,
									parameterTypes[i].getName(), paramDesc);
			}
		}
	}

	/**
     * Constructs an <CODE>MBeanOperationInfo</CODE> object.
     *
     * @param name The name of the method.
     *
     * @param description A human readable description of the operation.
     *
     * @param signature <CODE>MBeanParameterInfo</CODE> objects describing
	 * 				the parameters(arguments) of the method.
	 *
     * @param type The type of the method's return value.
     *
     * @param impact The impact of the method, one of
	 * 				<CODE>INFO, ACTION, ACTION_INFO, UNKNOWN</CODE>.
	 */
	public MBeanOperationInfo (String name,
								 String description,
								 MBeanParameterInfo[] signature,
								 String type,
								 int impact)
	{
		super(name, description);

		if(signature != null)
			this.signature = signature;
		else
			this.signature = new MBeanParameterInfo[0];

		this.type = type;
		this.impact = impact;
	}

	/**
	 * Returns the impact of the method, one of
	 * <CODE>INFO</CODE>, <CODE>ACTION</CODE>, <CODE>ACTION_INFO</CODE>,
	 * <CODE>UNKNOWN</CODE>.
	 *
	 * @return This returns the impact of the method
	 */
	public int getImpact()
	{
		return impact;
	}


	/**
	 * Returns the signature of the method, that is, information on
	 * the operations arguments.
	 *
	 * @return This returs an array of MBeanParameterInfo that is,
	 * 				information on the operations arguments.
	 */
	public MBeanParameterInfo[] getSignature()
	{
		return signature;
	}

	/**
	 * Returns the description of the method's return value.
	 *
	 * @return The return type of the method is returned as a String
	 */
	public String getReturnType()
	{
		return type;
	}

	/**
	 * Creates and returns a copy of this object.
	 *
	 * @return A duplicate copy of this object is created
	 */
	public Object clone()
	{
		MBeanParameterInfo[] clonedSig = new MBeanParameterInfo[signature.length];

		for(int i=0;i<signature.length;i++)
			clonedSig[i] = (MBeanParameterInfo)signature[i].clone();

		return new MBeanOperationInfo(getName(), getDescription(), clonedSig, type, impact);
	}

	/**
	 * Returns a human readable version of the MBeanOperationInfo instance
	 *
	 * @return  Human readable version of the MBeanOperationInfo instance is returned
	 */
	public String toString()
	{
		return (super.toString() +
				"\n Type = "+ type +
				"\n Impact = "+ convertImpact(impact) +
				"\n Signature = "+ getSignatureString());
	}

	//------------------------- Private methods ---------------------------//

	private String convertImpact(int impact)
	{
		if(impact == ACTION)
			return "ACTION";
		else if(impact == ACTION_INFO)
			return "ACTION_INFO";
		else if(impact == INFO)
			return "INFO";
		else if(impact == UNKNOWN)
			return "UNKNOWN";

		return "inValid";
	}

	private String getSignatureString()
	{
		if(signature == null)
			return "null";

		String toRet = "";

		for (int i=0;i<signature.length;i++)
			toRet = (toRet + " [param"+ (i+1) + " = " + signature[i].toString() + "]\n" );

		return toRet;
	}
}