/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management;

import java.io.Serializable;
import java.util.Vector;

/**
 * Provides an implementation of the {@link javax.management.NotificationFilter}
 * interface. The filtering is performed on the notification type attribute.
 * <P>
 * This class manages a list of enabled notification types.
 * A method allows users to enable/disable as many notification types as required.
 * <P>
 * Then, before sending a notification to a listener registered with a filter,
 * the notification broadcaster compares this notification type with all
 * notification types enabled by the filter. The notification will be sent to
 * the listener only if its filter enables this notification type.
 * <P>
 * Example:
 * <BLOCKQUOTE>
 * <PRE>
 * NotificationFilterSupport myFilter = new NotificationFilterSupport();
 * myFilter.enableType("my_example.my_type");
 * myBroadcaster.addListener(myListener, myFilter, null);
 * </PRE>
 * </BLOCKQUOTE>
 * The listener <CODE>myListener</CODE> will only receive notifications
 * the type of which equals/starts with "my_example.my_type".
 *
 * @see javax.management.NotificationBroadcaster#addNotificationListener
 */
public class NotificationFilterSupport
							implements NotificationFilter, Serializable
{
    /**
     * Vector that contains the enabled notification types.
     * The default value is an empty vector.
     */
	protected Vector enabledTypes = null;

	/**
	 * Creates a NotificationFilterSupport
	 * Default constructor
	 */
	public NotificationFilterSupport()
	{
		enabledTypes = new Vector();
	}

	/**
	 * Invoked before sending the specified notification to the listener.
	 * This filter compares the notification type with each enabled type.
	 * If the notification type matches one of the enabled type, the
	 * notification has to be sent to the listener and this method returns true.
	 *
     * @param notification The notification to be sent.
     *
     * @return True if the notification has to be sent to the listener, false otherwise.
	 */
	public boolean isNotificationEnabled(Notification notification)
	{
		if(notification == null)
			return false;

		String type = notification.getType();

		for(int i=0;i<enabledTypes.size();i++)
		{
			String str = (String)enabledTypes.elementAt(i);

			if(type.toLowerCase().startsWith(str.toLowerCase()))
				return true;
		}

		return false;
	}

	/**
	 * Disables all the notification types.
	 */
	public void disableAllTypes()
	{
		enabledTypes.removeAllElements();
	}

	/**
	 * Disables all the notifications whose type matches the specified prefix
	 * to be sent to the listener. If the specified prefix is not in the list
	 * of enabled notification types, this method has no effect.
	 *
	 * @param prefix The prefix.
	 */
	public void disableType(String prefix)
	{
		if(prefix == null)
			return;

		for(int i=0;i<enabledTypes.size();i++)
		{
			String str = (String)enabledTypes.elementAt(i);

			if(str.toLowerCase().startsWith(prefix.toLowerCase()))
				enabledTypes.removeElementAt(i);
		}
	}

	/**
	 * Enables all the notifications whose type matches the specified prefix
	 * to be sent to the listener. If the specified prefix is already in the
	 * list of enabled notification types, this method has no effect.
	 *
	 * @param prefix The prefix.
	 *
	 * @throws IllegalArgumentException The prefix parameter is null.
	 */
	public void enableType(String prefix)
										throws IllegalArgumentException
	{
		if(prefix == null)
			throw new IllegalArgumentException(prefix);

		boolean flag = false;

		for(int i=0;i<enabledTypes.size();i++)
		{
			String str = (String)enabledTypes.elementAt(i);

			if(prefix.toLowerCase().startsWith(str.toLowerCase()))
				flag = true;
			else if(str.toLowerCase().startsWith(prefix.toLowerCase()))
				enabledTypes.removeElementAt(i);
		}

		if(!flag)
			enabledTypes.add(prefix);
	}

	/**
	 * Gets all the enabled notification types for this filter.
	 *
	 * @return The list containing all the enabled notification types.
	 */
	public Vector getEnabledTypes()
	{
		return enabledTypes;
	}
}