/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management;

import java.io.Serializable;
import java.io.ObjectInputStream;
import java.io.ObjectInputStream.GetField;
import java.io.ObjectOutputStream;
import java.io.ObjectOutputStream.PutField;
import java.io.ObjectStreamField;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;

import com.adventnet.jmx.utils.Sorter;

/**
 * This class represents the object name of an MBean.
 * An instance of this class can be used to represent:
 * <ul>
 * <li> An object name
 * <li> An object name pattern, within the context of a query
 * </ul>
 */
public class ObjectName implements Serializable
{
    /**
     * Domain name.
     */
	private String domainName = null;

    private boolean noDomain = false;
    /** **/
    private Hashtable keyProperty = null;

    private String propertyString = null;

    /** **/
    private Sorter qsort = null;

    /** **/
    private boolean wildFlag = false;

	static final long serialVersionUID = 0xb41e7d55cfd55210L;

    /** **/
    private boolean pattern = false;

	private transient String cachedKeyList = null;
	private transient String toString = null;

	private static final ObjectStreamField[] serialPersistentFields =
			{
				new ObjectStreamField("domain", java.lang.String.class),
            	new ObjectStreamField("propertyList",java.util.Hashtable.class),
                new ObjectStreamField("propertyListString",java.lang.String.class),
                new ObjectStreamField("canonicalName",java.lang.String.class),
                new ObjectStreamField("pattern",java.lang.Boolean.TYPE),
                new ObjectStreamField("propertyPattern",java.lang.Boolean.TYPE),
			};


	/** **/
	private ObjectName()
	{
	    keyProperty = new Hashtable();
	    qsort = new Sorter();
	}

    /**
     * Allows an object name to be created from the given string. If the string
     * does not have the right format an MalformedObjectNameException should be
     * raised.
     *
     * @param name - A string representation of the object name.
     *
     * @throws MalformedObjectNameException - the string passed in parameter
     * 				does not have the right format.
     */
	public ObjectName(String name) throws MalformedObjectNameException
    {
	    this();

		try
    	{
            String props = null;

            if(name.startsWith(":"))
            	noDomain = true;

            if(name.equals("") || name == null)
			{
				noDomain = true;
                    name = "*:*";
            }

            if(name.indexOf(":") == -1)
			{
            	throw new MalformedObjectNameException("Invalid ObjectName");
            }

            propertyString = name.substring(name.indexOf(":") +1 ,name.length());

            StringTokenizer st = new StringTokenizer(name, ":",true);

			if(st.countTokens() > 3)
			{
            	throw new MalformedObjectNameException("Invalid ObjectName");
            }

            String temp = null;
            if(st.countTokens() == 2)
			{
                temp = st.nextToken();
                if(temp.equals(":"))
                {
					//Get the default domain from the MBeanserver
					ArrayList list = MBeanServerFactory.findMBeanServer(null);
					if(list != null)
					{
						if (list.toArray().length > 0)
						{
							MBeanServer server = (MBeanServer)list.toArray()[0];
							domainName = server.getDefaultDomain();
						}
						else
							domainName = "DefaultDomain";
					}
					else
                        domainName = "DefaultDomain";

                    props = st.nextToken();
                }
                else
				{
                    domainName = temp;
                    props = "*";
                }
            }

            if(st.countTokens() == 3)
			{
                temp = st.nextToken();
                if(temp.equals(":"))
				{
                	throw new MalformedObjectNameException("Invalid ObjectName");
                }
                else
				{
                     domainName = temp;
                     st.nextToken();
                     temp = st.nextToken();
                     if(temp.equals(":"))
					 {
                     	throw new MalformedObjectNameException("Invalid ObjectName");
                     }
                     props = temp;
                }
            }

			checkDomain(domainName);
            st = new StringTokenizer(props, ",",true);

            while (st.hasMoreTokens())
            {
                String key = null;
                String value = null;
                String prop = st.nextToken();

                /* For TCK. Consecutive comma(,) is not allowed in type seperation */
                if(prop.equals(","))
				{
                    try
					{
                    	prop = st.nextToken();
                    }
                    catch(NoSuchElementException ne)
					{
                        throw new MalformedObjectNameException("Invalid ObjectName");
                    }
                    if(prop.equals(","))
					{
                    	throw new MalformedObjectNameException("Invalid ObjectName");
                    }
                }

                if(prop.equals("*"))
                {
                    wildFlag = true;
                    continue;
                }
                if(prop.indexOf("=") == -1)
                {
                    throw new MalformedObjectNameException("Invalid ObjectName");
                }

                /* For TCK. More than one = in a type=val is wrong (e.g) type=r=t */
                StringTokenizer tok = new StringTokenizer(prop, "=",true);
                if(tok.countTokens() !=3 )
				{
                	throw new MalformedObjectNameException("Invalid ObjectName");
                }
                temp = null;

                while(tok.hasMoreTokens())
				{
                    temp = tok.nextToken();
                    if(temp.equals("="))
					{
						throw new MalformedObjectNameException("Invalid ObjectName");
                    }
                    else{
                        key = temp;
                        //this will be = for sure
                        tok.nextToken();
                        temp = tok.nextToken();
                        if(temp.equals("="))
						{
                        	throw new MalformedObjectNameException("Invalid ObjectName");
                        }
                        value = temp;
                    }
                }
                if(keyProperty.put(key, value) != null)
				{
                	throw new MalformedObjectNameException("Duplicate Key not allowed");
                }
            }
            toString();
        }
		catch(Exception e)
		{
        	throw new MalformedObjectNameException(e.getMessage());
        }
	}

	/**
	 * Allows an object name with several key properties to be created. If the
	 * parameters do not have the right format an MalformedObjectNameException
	 * should be raised.
	 *
	 * @param domain - The domain part of the object name.
	 *
	 * @param table - A hash table containing one or more search keys.
	 * 				Each search key is an attribute-value pair.
	 *
	 * @throws MalformedObjectNameException - the string passed in parameter
	 * 				does not have the right format.
	 */
	public ObjectName(String domain, Hashtable table) throws javax.management.MalformedObjectNameException
	{
	    this();

	    if(domain == null)
	    {
            /* Tck Compliance */
            throw new MalformedObjectNameException("Invalid ObjectName");
            //domain = "DefaultDomain";
	    }

	    if(domain.equals(""))
	    	noDomain = true;

	    this.domainName = domain;

	    if(table != null && table.size() > 0)
	    {
			Enumeration enum = table.keys();
	        String kkey = null;
			String value = null;

	        while(enum.hasMoreElements())
	        {
				kkey = (enum.nextElement()).toString();
				value = (table.get(kkey)).toString();
				propertyString = kkey+"="+value+",";
			}

			propertyString = propertyString.substring(0,propertyString.length() -1);

	        Enumeration keys = table.keys();
	        String key = null;

	        while(keys.hasMoreElements())
	        {
	        	key = keys.nextElement().toString();
	            Object val = table.get(key);
	            if(!(val instanceof String))
	            {
	            	throw new MalformedObjectNameException("Invalid ObjectName");
	            }
	            String str = (String)val;
	            if(str.equals(""))
	            {
	            	throw new MalformedObjectNameException("Invalid ObjectName");
	            }
				if(str.equals("temp"))
				{
					table.remove(key);
				}
	        }
	        keyProperty = table;
		}
	    else
	    {
	    	throw new MalformedObjectNameException("Atleast one Key Property should be present");
		}
		toString();
	}

    /**
     * Allows an object name to be created using only one key property. If the
     * parameters do not have the right format an MalformedObjectNameException
     * should be raised.
     *
     * @param domain - The domain part of the object name.
     *
     * @param key - The attribute in the key property of the object name.
     *
     * @param value - The value in the key property of the object name.
     *
     * @throws MalformedObjectNameException - the string passed in parameter
     * 				does not have the right format.
     */
    public ObjectName(String domain, String key, String value)
        								throws MalformedObjectNameException
    {
		this();

		if(domain == null)
		{
	        /** TCK compliance */
	        throw new MalformedObjectNameException("Invalid ObjectName");
	        //this.domainName = "DefaultDomain";
		}

        this.domainName = domain;

        if(domain.equals(""))
        	noDomain = true;

		if(key == null || key.equals("") || value == null || value.equals(""))
		{
			throw new MalformedObjectNameException("Invalid ObjectName");
		}

		keyProperty.put(key, value);
		propertyString = key+"="+value;

		toString();
	}

    /**
     * Returns the domain part.
     *
     * @return This returns the domain name
     */
    public String getDomain()
    {
		if(noDomain)
			return "";

        return domainName;
    }

    /**
     * Obtains the value of a key property in a key property.
     *
     * @param property - The property whose value is to be obtained.
     *
     * @return The value of the property.
     */
    public String getKeyProperty(String property)
    {
		return (String)keyProperty.get(property);
    }

    /**
     * Returns a pointer to the list of key properties.
     *
     * @return This returns a HashTable of key properties
     */
    public Hashtable getKeyPropertyList()
    {
       return keyProperty;
    }

    /**
     * Checks if a name to be used is a pattern for a query.
     *
     * @return True if the name is a pattern, otherwise false.
     */
    public boolean isPattern()
    {
        String temp = toString();

        if(temp.indexOf("*") != -1 || temp.indexOf("?") != -1)
                pattern = true;

        return pattern;
    }

    /**
     * Returns a string representation of the object name.
     *
     * @return This returns the string representation of the oject name
     */
    public String toString()
    {
		if(toString != null) {
			return toString;
		}

        StringBuffer buff = new StringBuffer();
        buff.append(domainName + ":");
       	buff.append(getKeyList());

        toString = buff.toString();
        return toString;
    }

    /**
     * Compares the current object name with another object name.
     *
     * @param object - The object name that the current object name is to be
     * 				compared with.
     *
     * @return True if the two object names are equal, otherwise false.
     */
    public boolean equals(Object object)
    {
		return toString().equals(object.toString());
    }

    /**
     * Returns a hash code for this object name.
     *
     * @return This returns a hash code of type int for this object name
     */
    public int hashCode()
    {
        return toString().hashCode();
    }

    /**
     * Returns the canonical form of the name; that is, a string representation
	 * where the properties are sorted in lexical order.
	 *
     * @return  Returns the canonical form of the name; that is, a string
	 * 				representation where the properties are sorted in lexical order.
     */
    public java.lang.String getCanonicalName()
    {
        if(noDomain)
        	return ":"+getKeyList();

        return domainName+":"+getKeyList();
    }

    /**
     * Returns a string representation of the list of key properties specified at creation
     * time.
     * @return This returns a string representation of the list of key properties
     */
    public java.lang.String getKeyPropertyListString()
    {
		return propertyString;
    }

    /**
     * Returns a string representation of the list of key properties in which
	 * the key properties are sorted in lexical order. This is used in
	 * lexicographic comparisons performed in order to select MBeans based on
	 * their key property list.
	 *
     * @return This returns a string representation of the list of key properties
	 * 				in which the key properties are sorted in lexical order.
     */
    public java.lang.String getCanonicalKeyPropertyListString()
    {
		return getKeyList();
	}

    /**
     * Checks whether the object name is a pattern on the key properties.
     *
     * @return true if the name is a pattern, otherwise false
     */
    public boolean isPropertyPattern()
    {
		String propertyString = getKeyPropertyListString();

		if(propertyString.indexOf("*") != -1)
		    return true;
		else
		    return false;
	}

	//-------------------------- Private method --------------------------//

	private String getKeyList()
	{
		if(cachedKeyList != null) {
			return cachedKeyList;
		}

	    StringBuffer buff = new StringBuffer();

	    String[] keys = new String[keyProperty.size()];
	    Object[] values = new Object[keyProperty.size()];

	    int i=0;

	    for(Enumeration e = keyProperty.keys(); e.hasMoreElements();i++)
	    {
            keys[i] = (String)e.nextElement();
            values[i] = keyProperty.get(keys[i]);
	    }

	    try
		{
            Object[][] obj = {values};
            getSorter().sort(keys, obj);
	    }
		catch(Exception e)
		{
	        System.err.println("Exception = "+e);
			//e.printStackTrace();
	    }

	    for(i=0; i<keyProperty.size(); i++)
	    {
			if(i==0)
			    buff.append(keys[i] + "=" + values[i].toString());
			else
			    buff.append("," + keys[i] + "=" + values[i].toString());
	    }

	    if(wildFlag)
	    {
            if(keyProperty.size() == 0)
				buff.append("*");
            else
                buff.append(",*");
	    }

		cachedKeyList = buff.toString();
        return cachedKeyList;
	}

	private Sorter getSorter()
	{
		if(qsort == null)
			qsort = new Sorter();

		return qsort;
	}

	private void checkDomain(String domain) throws MalformedObjectNameException
	{
	    if (domain.indexOf(':') >= 0)
			throw new MalformedObjectNameException("Domain cannot contain ':' characters");
	    if (domain.indexOf(',') >= 0)
			throw new MalformedObjectNameException("Domain cannot contain ',' characters");
	    if (domain.indexOf('=') >= 0)
			throw new MalformedObjectNameException("Domain cannot contain '=' characters");
	}

	private void readObject(ObjectInputStream objectinputstream)
	    						throws IOException, ClassNotFoundException
	{
	    ObjectInputStream.GetField getfield = objectinputstream.readFields();

	    try
	    {
	        domainName=(String)getfield.get("domain", "");
	        keyProperty=(Hashtable)getfield.get("propertyList",null);
	        propertyString=(String)getfield.get("propertyListString","");
	        pattern=getfield.get("pattern",false);
			wildFlag = isPropertyPattern();
			toString();
	    }
	    catch(Exception exception)
	    {
			exception.printStackTrace();
	    }
	}

	private void writeObject(ObjectOutputStream objectoutputstream)
														throws IOException
	{
		ObjectOutputStream.PutField putfield = objectoutputstream.putFields();
		putfield.put("domain", domainName);
		putfield.put("propertyList", getKeyPropertyList());
		putfield.put("propertyListString", getKeyPropertyListString());
		putfield.put("canonicalName", getCanonicalName());
		putfield.put("pattern", isPattern());
		putfield.put("propertyPattern", isPropertyPattern());
		objectoutputstream.writeFields();
	}
}//End of class ObjectName