/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management;


/**
 * This Query class supports construction of query object constraints.
 * The static methods provided return query expressions that may be used in
 * listing and enumerating MBeans. Individual constraint constructing methods
 * allow only appropriate types as arguments. Composition of calls can
 * construct arbitrary nestings of constraints, as the following
 * example illustrates:
 * QueryExp exp = Query.and(Query.gt(Query.attr("age"),Query.value(5)),
 * 					Query.match(Query.attr("name"), Query.value("John")));
 */
public class Query
{
	public static final int DIV = 1;

	public static final int EQ = 2;

	public static final int GE = 3;

	public static final int GT = 4;

	public static final int LE = 5;

	public static final int LT = 6;

	public static final int MINUS = 7;

	public static final int PLUS  = 8;

	public static final int TIMES = 9;


	static final int MATCH = 10;

	static final int AND = 11;

	static final int OR = 12;

	static final int BET = 13;

	static final int NOT = 14;

	static final int IN = 15;

	static final int INITIAL = 16;

	static final int ANY = 17;

	static final int FINAL = 18;


	/**
	 * Constructs a query object.
	 */
	public Query()
	{
	}

	/**
	 * Returns a new attribute expression.
	 *
	 * @param name - The name of the attribute.
	 *
	 * @return An attribute expression for the attribute named name.
	 */
	public static AttributeValueExp attr(String name)
	{
		return new AttributeValueExp(name);
	}

	/**
	 * Returns a new qualified attribute expression.
	 *
	 * @param className - The name of the class possessing the attribute
	 *
	 * @param name - The name of the attribute
	 *
	 * @return An attribute expression for the attribute named name.
	 */
	public static AttributeValueExp attr(String className, String name)
	{
		return new AttributeValueExp(className, name);
	}

	/**
	 * Returns a new class attribute expression which can be used in any
	 * Query call that expects a ValueExp.
	 *
	 * @return A class attribute expression.
	 */
	public static AttributeValueExp classattr()
	{
		return new AttributeValueExp();
	}

	/**
	 * Returns a numeric value expression that can be used in any Query
	 * call that expects a ValueExp.
	 *
	 * @param val - An int value
	 *
	 * @return A ValueExp object containing the argument.
	 */
	public static ValueExp value(int val)
	{
		return new AttributeValueExp(null, new Integer(val));
	}

	/**
	 * Returns a numeric value expression that can be used in any Query
	 * call that expects a ValueExp.
	 *
	 * @param val - A long value
	 *
	 * @return A ValueExp object containing the argument.
	 */
	public static ValueExp value(long val)
	{
		return new AttributeValueExp(null, new Long(val));
	}

	/**
	 * Returns a numeric value expression that can be used in any Query
	 * call that expects a ValueExp.
	 *
	 * @param val - A float value
	 *
	 * @return A ValueExp object containing the argument.
	 */
	public static ValueExp value(float val)
	{
		return new AttributeValueExp(null, new Float(val));
	}

	/**
	 * Returns a numeric value expression that can be used in any Query
	 * call that expects a ValueExp.
	 *
	 * @param val - A double value
	 *
	 * @return A ValueExp object containing the argument.
	 */
	public static ValueExp value(double val)
	{
		return new AttributeValueExp(null, new Double(val));
	}

	/**
	 * Returns a boolean value expression that can be used in any Query
	 * call that expects a ValueExp.
	 *
	 * @param val - A boolean value
	 *
	 * @return A ValueExp object containing the argument.
	 */
	public static ValueExp value(boolean val)
	{
		return new AttributeValueExp(null, new Boolean(val));
	}

	/**
	 * Returns a numeric value expression that can be used in any Query
	 * call that expects a ValueExp.
	 *
	 * @param val - An instance of Number
	 *
	 * @return A ValueExp object containing the argument.
	 */
	public static ValueExp value(Number val)
	{
		return new AttributeValueExp(null, val);
	}

	/**
	 * Returns a new string expression.
	 *
	 * @param val - The string value
	 *
	 * @return A ValueExp object containing the string argument.
	 */
	public static StringValueExp value(String val)
	{
		return new StringValueExp(val);
	}

	/**
	 * Returns a binary expression representing the sum of two numeric values
	 * or the concatenation of two string values.
	 *
	 * @param v1 - The first '+' operand.
	 *
	 * @param v2 - The second '+' operand.
	 *
	 * @return A ValueExp representing the sum or concatenation of the two arguments
	 */
	public static ValueExp plus (ValueExp v1, ValueExp v2)
	{
		if(v1 instanceof AttributeValueExp && v2 instanceof AttributeValueExp)
			return new AttributeValueExp((AttributeValueExp)v1, (AttributeValueExp)v2, PLUS);

		if(v1 instanceof StringValueExp && v2 instanceof StringValueExp)
			return new StringValueExp((StringValueExp)v1, (StringValueExp)v2, PLUS);

		return null;
	}

	/**
	 * Returns a binary expression representing the difference of two
	 * numeric values.
	 *
	 * @param v1 - The first '-' operand.
	 *
	 * @param v2 - The second '-' operand.
	 *
	 * @return A ValueExp representing the difference of two arguments.
	 */
	public static ValueExp minus (ValueExp v1, ValueExp v2)
	{
		if(v1 instanceof AttributeValueExp && v2 instanceof AttributeValueExp)
			return new AttributeValueExp((AttributeValueExp)v1, (AttributeValueExp)v2, MINUS);

		return null;
	}

	/**
	 * Returns a binary expression representing the product of two numeric values.
	 *
	 * @param v1 - The first '*' operand.
	 *
	 * @param v2 - The second '*' operand.
	 *
	 * @return A ValueExp representing the product.
	 */
	public static ValueExp times (ValueExp v1, ValueExp v2)
	{
		if(v1 instanceof AttributeValueExp && v2 instanceof AttributeValueExp)
			return new AttributeValueExp((AttributeValueExp)v1, (AttributeValueExp)v2, TIMES);

		return null;
	}

	/**
	 * Returns a binary expression representing the quotient of two numeric values.
	 *
	 * @param v1 - The first '/' operand.
	 *
	 * @param v2 - The second '/' operand.
	 *
	 * @return A ValueExp representing the quotient of two arguments.
	 */
	public static ValueExp div (ValueExp v1, ValueExp v2)
	{
		if(v1 instanceof AttributeValueExp && v2 instanceof AttributeValueExp)
			return new AttributeValueExp((AttributeValueExp)v1, (AttributeValueExp)v2, DIV);

		return null;
	}

	/**
	 * Returns a query expression that represents a "greater than" constraint
	 * on two values.
	 *
	 * @param v1 - A value expression
	 *
	 * @param v2 - Another value expression
	 *
	 * @return A "greater than" constraint on the arguments.
	 */
	public static QueryExp gt (ValueExp v1, ValueExp v2)
	{
		return new QueryExpSupport(v1, v2, GT);
	}

	/**
	 * Returns a query expression that represents a matching constraint on a
	 * string argument. The matching syntax is consistent with file globbing:
	 * Supports "?", "*", "[", each of which may be escaped with "\";
	 * Character classes may use "!" for negation and "-" for range.
	 * (* for any character sequence ? for a single arbitrary character [...]
	 * for a character sequence). For example: a*b?c would match a string
	 * starting with the character a, followed by any number of characters,
	 * followed by a b, any single character, and a c.
	 *
	 * @param a - An attribute expression
	 *
	 * @param s - A string value expression representing a matching constraint
	 *
	 * @return A query expression that represents the matching constraint
	 * 				on the string argument.
	 */
	public static QueryExp match (AttributeValueExp a, StringValueExp s)
	{
		return new QueryExpSupport(a, s, MATCH);
	}

	/**
	 * Returns a query expression that represents a "greater than or equal to"
	 * constraint on two values.
	 *
	 * @param v1 - A value expression
	 *
	 * @param v2 - Another value expression
	 *
	 * @return A "greater than or equal to" constraint on the arguments.
	 */
	public static QueryExp geq (ValueExp v1, ValueExp v2)
	{
		return new QueryExpSupport(v1, v2, GE);
	}

	/**
	 * Returns a query expression that represents a "less than or equal to"
	 * constraint on two values.
	 *
	 * @param  v1 - A value expression
	 *
	 * @param  v2 - Another value expression
	 *
	 * @return A "less than or equal to" constraint on the arguments.
	 */
	public static QueryExp leq (ValueExp v1, ValueExp v2)
	{
		return new QueryExpSupport(v1, v2, LE);
	}

	/**
	 * Returns a query expression that represents a "less than" constraint
	 * on two values.
	 *
	 * @param v1 - A value expression
	 *
	 * @param v2 - Another value expression
	 *
	 * @return A "less than" constraint on the arguments.
	 */
	public static QueryExp lt (ValueExp v1, ValueExp v2)
	{
		return new QueryExpSupport(v1, v2, LT);
	}

	/**
	 * Returns a query expression that represents an equality constraint
	 * on two values.
	 *
	 * @param v1 - A value expression
	 * @param v2 - Another value expression
	 *
	 * @return A "equal to" constraint on the arguments.
	 */
	public static QueryExp eq (ValueExp v1, ValueExp v2)
	{
		return new QueryExpSupport(v1, v2, EQ);
	}

	/**
	 * Returns a query expression that represents the constraint that one
	 * value is between two other values.
	 *
	 * @param v1 - A value expression that is "between" v2 and v3
	 *
	 * @param v2 - Value expression that represents a boundary of the constraint
	 *
	 * @param v3 - Value expression that represents a boundary of the constraint
	 *
	 * @return The constraint that v1 lies between v2 and v3.
	 */
	public static QueryExp between (ValueExp v1, ValueExp v2, ValueExp v3)
	{
		return new QueryExpSupport(v1, v2, v3, BET);
	}

	/**
	 * Returns an expression constraining a value to be one of an explicit list.
	 *
	 * @param val - A value to be constrained
	 *
	 * @param valueList - An array of ValueExps
	 *
	 * @return A QueryExp that represents the constraint.
	 */
	public static QueryExp in (ValueExp va1, ValueExp[] valueList)
	{
		return new QueryExpSupport(va1, valueList, IN);
	}

	/**
	 * Returns a query expression that represents a matching constraint on a
	 * string argument. The value must start with the given string value.
	 *
	 * @param a - An attribute expression
	 *
	 * @param s - A string value expression representing the beginning
	 * 				of the string value
	 *
	 * @return The constraint that a matches s.
	 */
	public static QueryExp initialSubString (AttributeValueExp a, StringValueExp s)
	{
		return new QueryExpSupport(a, s, INITIAL);
	}

	/**
	 * Returns a query expression that represents a matching constraint on a
	 * string argument. The value must contain the given string value.
	 *
	 * @param a - An attribute expression
	 *
	 * @param s - A string value expression representing the substring.
	 *
	 * @return The constraint that a matches s.
	 */
	public static QueryExp anySubString (AttributeValueExp a, StringValueExp s)
	{
		return new QueryExpSupport(a, s, ANY);
	}

	/**
	 * Returns a query expression that represents a matching constraint on a
	 * string argument. The value must contain the given string value.
	 *
	 * @param a - An attribute expression
	 *
	 * @param s - A string value expression representing the end of the string value
	 *
	 * @return The constraint that a matches s.
	 */
	public static QueryExp finalSubString (AttributeValueExp a, StringValueExp s)
	{
		return new QueryExpSupport(a, s, FINAL);
	}

	/**
	 * Returns a query expression that is the conjunction of two other query
	 * expressions.
	 *
	 * @param q1 - A query expression
	 *
	 * @param q2 - Another query expression
	 *
	 * @return The conjunction of the two arguments.
	 */
	public static QueryExp and (QueryExp q1, QueryExp q2)
	{
		return new QueryExpSupport(q1, q2, AND);
	}

	/**
	 * Returns a query expression that is the disjunction of two other
	 * query expressions.
	 *
	 * @param q1 - A query expression
	 *
	 * @param q2 - Another query expression
	 *
	 * @return The disjunction of the two arguments.
	 */
	public static QueryExp or (QueryExp q1, QueryExp q2)
	{
		return new QueryExpSupport(q1, q2, OR);
	}

	/**
	 * Returns a constraint that is the negation of its argument.
	 *
	 * @param queryExp - The constraint to negate
	 *
	 * @return A negated constraint.
	 */
	public static QueryExp not (QueryExp queryExp)
	{
		return new QueryExpSupport(queryExp, NOT);
	}
}//End of class Query