/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.modelmbean;

import java.io.ObjectInputStream;
import java.io.ObjectInputStream.GetField;
import java.io.ObjectOutputStream;
import java.io.ObjectOutputStream.PutField;
import java.io.ObjectStreamField;
import java.io.IOException;
import java.util.HashMap;
import java.util.StringTokenizer;

import javax.management.Descriptor;
import javax.management.MBeanException;
import javax.management.RuntimeOperationsException;

import com.adventnet.utils.jmx.Utilities;

/**
 * This class represents the metadata set for a ModelMBean element. A
 * descriptor is part of the ModelMBeanInfo, ModelMBeanNotificationInfo,
 * ModelMBeanAttributeInfo, ModelMBeanConstructorInfo, and ModelMBeanParameterInfo.
 * <p>
 * A descriptor consists of a collection of fields. Each field is in
 * fieldname=fieldvalue format. Field names are not case sensitive, case will
 * be preserved on field values.
 * <p>
 * All field names and values are not predefined. New fields can be defined
 * and added by any program. Some fields have been predefined for consistency
 * of implmentation and support by the ModelMBeanInfo ModelMBean*Info, and
 * ModelMBean classes.
 * <p>
 * By default the descriptor store size is 20 fields. This is configurable.
 */
public class DescriptorSupport implements Descriptor
{
	public String currClass = "DescriptorSupport";

	static final long serialVersionUID = 8071560848919417985L;

	HashMap store = null;

	int DEFAULT_NUM_FIELDS = 20;

    static String NAME = "name";
    static String DESCRIPTOR_TYPE = "descriptorType";
    static String VALUE = "value";
    static String DEFAULT = "default";
    static String DISPLAY_NAME = "displayName";
    static String GET_METHOD = "getMethod";
    static String SET_METHOD = "setMethod";
    static String PROTOCOL_MAP = "protocolMap";
    static String PERSIST_POLICY = "persistPolicy";
    static String PERSIST_PERIOD = "persistPeriod";
    static String CURRENCY_TIME_LIMIT = "currencyTimeLimit";
    static String LAST_UPDATED_TIMESTAMP = "lastUpdatedTimeStamp";
    static String ITERABLE = "iterable";
    static String VISIBILITY = "visibility";
    static String PRESENTATION_STRING = "presentationString";

    static String ROLE = "role";
    static String CLASS = "class";

    static String SEVERITY = "severity";
    static String MESSAGE_ID = "messageID";
    static String MESSAGE_TEXT = "messageText";
    static String LOG = "log";
    static String LOG_FILE = "logfile";

    static String TARGET_OBJECT = "targetObject";
    static String TARGET_TYPE = "targetObjectType";
    static String LAST_RETURNED_VALUE = "lastReturnedValue";

	private static final ObjectStreamField[] serialPersistentFields =
		{
			new ObjectStreamField("descriptor", java.util.HashMap.class),
			new ObjectStreamField("currClass", java.lang.String.class)
		};

    /**
     * Descriptor default constructor.
     */
    public DescriptorSupport()
    {
		store = new HashMap(DEFAULT_NUM_FIELDS);
    }

    /**
     * Descriptor constructor. Takes as parameter the initial capacity of the
	 * Map that stores the descriptor fields.
	 *
     * @param initNumFields The initial capacity of the Map that stores
     * 				the descriptor fields.
     */
    public DescriptorSupport(int initNumFields)
                    		throws MBeanException, RuntimeOperationsException
    {
		if(initNumFields <= 0)
			throw new RuntimeOperationsException(
				new RuntimeException("Wrong initNumFields to DescriptorSupport"));

		store = new HashMap(initNumFields);
    }

    /**
     * Descriptor constructor taking a Descriptor as parameter. Creates a new
	 * descriptor initialized to the values of the descriptor passed in parameter.
	 *
     * @param inDescr the descriptor to be used to initialize the constructed
     * 				descriptor. If it is null or contains no descriptor
	 * 				fields, an empty Descriptor will be created.
	 *
     * @exception RuntimeOperationsException for illegal value for inDescr.
	 * 				inDescr cannot be null. If the descriptor fails for any
	 * 				reason, this exception will be thrown.
     */
    public DescriptorSupport(DescriptorSupport inDescr)
    {
		if(inDescr == null)
        	throw new RuntimeOperationsException(
					new RuntimeException("invalid DescriptorSupport"));

		store = new HashMap(inDescr.store);
    }

    /**
     * Descriptor constructor taking an XML String. This method is part of
     * this implementation and not required by the JMX Specification.
     *
     * @param inStr - An XML-formatted string used to populate this Descriptor
     */
    public DescriptorSupport(String inStr)
			throws MBeanException, RuntimeOperationsException, XMLParseException
    {
		if(inStr == null)
		{
		    throw new RuntimeOperationsException(new IllegalArgumentException(
							"The String parameter passed should not be null"));
		}

		try
		{
		    ModelMBeanInfo info = Utilities.convertXmlToModelMBeanInfo(inStr);
		}
		catch(Exception e)
		{
		    throw new XMLParseException();
		}
    }

    /**
     * Constructor taking field names and field values. The array and array
	 * elements cannot be null.
	 *
     * @param fieldNames String array of field names. No elements of this
	 * 				array can be null.
	 *
     * @param fieldValues Object array of the corresponding field values.
	 * 				Elements of the array can be null. The fieldValue must be
	 * 				valid for the fieldName.
	 * 				Note:
	 * 				array sizes of parameters should match. If both arrays
	 * 				are null or empty, then an empty descriptor is created.
	 * 				No exception is thrown.
	 *
     * @exception RuntimeOperationsException for illegal value for field
	 * 				Names or field Values. The array lengths must be equal.
	 * 				If the descriptor construction fails for any reason,
	 * 				this exception will be thrown.
     */
    public DescriptorSupport(String[] fieldNames, Object[] fieldValues)
              							throws RuntimeOperationsException
    {
        this();

        if(fieldNames == null || fieldValues == null ||
           (fieldNames.length != fieldValues.length))
                throw new RuntimeOperationsException(new RuntimeException(
							"Invalid fieldNames|fieldValues as arguments"));

        for(int i=0;i<fieldNames.length;i++)
        {
			if(fieldNames[i] == null)
				throw new RuntimeOperationsException(new RuntimeException(
											"FieldName cannot be null"));
        }

        for(int i=0;i<fieldValues.length;i++)
        {
			if(fieldValues[i] == null)
            	throw new RuntimeOperationsException(new RuntimeException(
											"FieldValue cannot be null"));
        }

        for(int i=0;i<fieldNames.length;i++)
                store.put(fieldNames[i].toLowerCase(), fieldValues[i]);
	}

    /**
     * Constructor taking fields in the fieldName=fieldValue format.
     *
     * @param fields String array of with each element containing a field name
	 * 				and value. If this array is null or empty, then the
	 * 				default constructor will be executed. Null strings or
	 * 				empty strings will be ignored.
	 * 				Note:
	 * 				each string should be of the form fieldName=fieldValue,
	 * 				with no imbedded blanks or other punctuation.
	 *
     * @exception RuntimeOperationsException for illegal value for field Names
	 * 				or field Values. The field must contain an "=".
	 * 				"=fieldValue", "fieldName", and "fieldValue" are illegal.
	 * 				FieldName cannot be null. "fieldName=" will cause the
	 * 				value to be null. If the descriptor construction fails
	 * 				for any reason, this exception will be thrown.
     */
    public DescriptorSupport(String[] fields)
    {
		this();

		if(fields == null)
		    throw new RuntimeOperationsException(
		            new RuntimeException("Invalid fields as arguments"));

		String[] fieldNames = new String[fields.length];
		String[] fieldValues = new String[fields.length];

		try
		{
		    for(int i=0;i<fields.length;i++)
		    {
	            StringTokenizer strTok = new StringTokenizer(fields[i], "=");
	            fieldNames[i] = strTok.nextToken();
	            fieldValues[i] = strTok.nextToken();
		    }
		}
		catch(Exception e)
		{
		    throw new RuntimeOperationsException(
		            new RuntimeException("Invalid fields as arguments"));
		}

		for(int i=0;i<fieldNames.length;i++)
		    store.put(fieldNames[i].toLowerCase(), fieldValues[i]);
    }

    /**
     * Returns the value for a specific fieldname.
     *
     * @param fieldName The field name in question; if not found, null is returned.
     *
     * @return This returns the field name of type Object
     *
     * @exception RuntimeOperationsException for illegal value for field name.
     */
    public Object getFieldValue(String fieldName)
										throws RuntimeOperationsException
    {
        if(fieldName == null)
            throw new RuntimeOperationsException(
                    new RuntimeException("Invalid fieldName"));

        return store.get(fieldName.toLowerCase());
    }

    /**
     * Sets the value for a specific fieldname. The field value will be
	 * validated before it is set.  If it is not valid, then an exception will
	 * be thrown. This will modify an existing field or add a new field.
	 *
     * @param  fieldName: The field name to be set. Cannot be null or empty.
     *
     * @param  fieldValue: The field value to be set for the field name. Can be null.
     *
     * @exception RuntimeOperationsException for illegal value for
	 * 				field name or field value.
     */
    public void setField(String fieldName, Object fieldValue)
											throws RuntimeOperationsException
    {
		if(fieldName == null )
		    throw new RuntimeOperationsException(
		            new RuntimeException("Invalid fieldName|fieldValue"));

		store.put(fieldName.toLowerCase(), fieldValue);
    }

    /**
     * Returns all of the  fields contained in this descriptor as a string array.
     *
     * @return String array of fields in the format fieldName=fieldValue.
	 * 				If the value of a field is not a String, then the toString
	 * 				method will be called on it and the returned value used
	 * 				as the value for the field in the returned array. Object
	 * 				values which are not Strings will be enclosed in parens.
	 * 				If the descriptor is empty, you will get an empty array.
     */
    public String[] getFields()
    {
		Object[] fieldNames = store.keySet().toArray();
		String[] toRet = new String[fieldNames.length];

		for(int i=0;i<fieldNames.length;i++)
		{
		     String field = (String)fieldNames[i];

		     Object fieldValue = store.get(field.toLowerCase());
		     if(fieldValue != null)
		     {
				if(fieldValue instanceof String)
				     field = field + "=" + (String)fieldValue;
				else
				     field = field + "=(" + fieldValue.toString() + ")";
		     }
		     else
		     {
	             field = field + "=" + "null";
		     }

		     toRet[i] = field;
		}

		return toRet;
    }

    /**
     * Returns all the fields names in the descriptor.
     *
     * @return String array of fields names. If the descriptor is empty,
	 * 				you will get an empty array.
     */
    public String[] getFieldNames()
    {
        Object[] objArr = store.keySet().toArray();
        String[] toRet = new String[objArr.length];

        for(int i=0;i<objArr.length;i++)
			toRet[i] = (String)objArr[i];

        return toRet;
    }

    /**
     * Returns all the field values in the descriptor as an array of Objects.
	 * The retuned values are in the same order as the fieldNames String array
	 * parameter.
	 *
     * @param fieldNames String array of the names of the fields that the
	 * 				values should be returned for.  If the array is empty then
	 * 				an empty array will be returned.  If the array is 'null'
	 * 				then all values will be returned.  If a field name in the
	 * 				array does not exist, then null is returned for the
	 * 				matching array element being returned.
	 *
     * @return Object array of field values. If the descriptor is empty,
	 * 				you will get an empty array.
     */
    public Object[] getFieldValues(String[] fieldNames)
    {
		if(fieldNames == null)
		    return store.values().toArray();

		Object[] fieldValues = new Object[fieldNames.length];

		for(int i=0;i<fieldNames.length;i++)
		    fieldValues[i] = store.get(fieldNames[i].toLowerCase());

		return fieldValues;
    }

    /**
    * Removes a field from the descriptor
    *
    * @param fieldName String name of the field to be removed. If the field
	* 				is not found no exception is thrown.
    */
    public void removeField(String fieldName)
    {
		store.remove(fieldName);
    }

    /**
     * Sets all Fields in the list to the new value in with the same index in
	 * the fieldValue array.  Array sizes must match. The field value will be
	 * validated before it is set. If it is not valid, then an exception will
	 * be thrown. If the arrays are empty, then no change will take effect.
	 *
     * @param fieldNames String array of field names. The array and array
	 * 				elements cannot be null.
	 *
     * @param fieldValues Object array of the corresponding field values.
	 * 				The array cannot be null. Elements of the array can be null.
	 *
     * @exception RuntimeOperationsException for illegal value for field Names
	 * 				or field Values. Neither can be null.  The array lengths
	 * 				must be equal. If the descriptor construction fails for
	 * 				any reason, this exception will be thrown.
     */
    public void setFields(String[] fieldNames, Object[] fieldValues)
											throws RuntimeOperationsException
    {
		if(fieldNames == null || fieldValues == null ||
				(fieldNames.length != fieldValues.length))
		    throw new RuntimeOperationsException(new RuntimeException(
								"Invalid fieldNames|fieldValues as arguments"));

		for(int i=0;i<fieldNames.length;i++)
		{
		    if(fieldNames[i] == null)
				throw new RuntimeOperationsException(new RuntimeException(
												"FieldName cannot be null"));
		}

		for(int i=0;i<fieldValues.length;i++)
		{
		    if(fieldValues[i] == null)
				throw new RuntimeOperationsException(new RuntimeException(
												"FieldValue cannot be null"));
		}

		for(int i=0;i<fieldNames.length;i++)
		    store.put(fieldNames[i].toLowerCase(), fieldValues[i]);
    }

	/**
	 * Returns a new Descriptor which is a duplicate of the Descriptor.
	 *
	 * @return This returns a duplicate copy of the Descriptor
	 *
	 * @exception RuntimeOperationsException for illegal value for field Names
	 * 				or field Values. If the descriptor construction fails for
	 * 				any reason, this exception will be thrown.
	 */
	public Object clone() throws RuntimeOperationsException
	{
		Descriptor clonedDescr = new DescriptorSupport();

		String[] fieldNames = getFieldNames();

		for(int i=0;i<fieldNames.length;i++)
		{
		    String fieldName = fieldNames[i];
		    Object fieldValue = getFieldValue(fieldName);

		    if(fieldValue instanceof Descriptor)
				fieldValue = ((Descriptor)fieldValue).clone();

		    clonedDescr.setField(fieldName, fieldValue);
		}

		return clonedDescr;
	}

    /**
     * Returns true if fieldValues are checked to be sure they are legal for
	 * the fieldNames.
	 *
     * @return This returns true if the fieldValues checked are valid otherwise false
     *
     * @exception RuntimeOperationsException If the validity checking fails
	 * 				for any reason, this exception will be thrown.
     */
    public boolean isValid() throws RuntimeOperationsException
    {
		Object[] fieldNames = store.keySet().toArray();

		for(int i=0;i<fieldNames.length;i++)
		{
		     String field = (String)fieldNames[i];
		     Object fieldValue = store.get(field.toLowerCase());

			if(field.equalsIgnoreCase(DESCRIPTOR_TYPE))
			    checkDescriptorType(DESCRIPTOR_TYPE, fieldValue);

			else if(field.equalsIgnoreCase(DISPLAY_NAME))
			    checkIsString(DISPLAY_NAME, fieldValue);

			else if(field.equalsIgnoreCase(GET_METHOD))
			    checkIsString(GET_METHOD, fieldValue);

			else if(field.equalsIgnoreCase(SET_METHOD))
			    checkIsString(SET_METHOD, fieldValue);

			else if(field.equalsIgnoreCase(PROTOCOL_MAP))
			    checkIsDescriptor(PROTOCOL_MAP, fieldValue);

			else if(field.equalsIgnoreCase(PERSIST_POLICY))
			    checkPersistPolicy(PERSIST_POLICY, fieldValue);

			else if(field.equalsIgnoreCase(CURRENCY_TIME_LIMIT))
			    checkIsInt(CURRENCY_TIME_LIMIT, fieldValue);

			else if(field.equalsIgnoreCase(LAST_UPDATED_TIMESTAMP))
			    checkIsInt(LAST_UPDATED_TIMESTAMP, fieldValue);

			else if(field.equalsIgnoreCase(ITERABLE))
			    checkIsBoolean(ITERABLE, fieldValue);

			else if(field.equalsIgnoreCase(VISIBILITY))
			    checkVisibility(VISIBILITY, fieldValue);

			else if(field.equalsIgnoreCase(PRESENTATION_STRING))
			    checkIsString(PRESENTATION_STRING, fieldValue);

			else if(field.equalsIgnoreCase(ROLE))
			    checkRole(ROLE, fieldValue);

			else if(field.equalsIgnoreCase(CLASS))
			    checkIsString(CLASS, fieldValue);

			else if(field.equalsIgnoreCase(SEVERITY))
			    checkSeverity(SEVERITY, fieldValue);

			else if(field.equalsIgnoreCase(MESSAGE_ID))
			    checkIsString(MESSAGE_ID, fieldValue);

			else if(field.equalsIgnoreCase(MESSAGE_TEXT))
			    checkIsString(MESSAGE_TEXT, fieldValue);

			else if(field.equalsIgnoreCase(LOG))
			    checkIsBoolean(LOG, fieldValue);

			else if(field.equalsIgnoreCase(LOG_FILE))
			    checkIsString(LOG_FILE, fieldValue);

			else if(field.equalsIgnoreCase(TARGET_TYPE))
			    checkTargetType(TARGET_TYPE, fieldValue);
		}

		return true;
    }

    /**
     * This method returns the XML which represents the descriptor.
     *
     * @return an XML String representing the descriptor.The XML string will
	 * 				be formatted for each field with " NAME=name
	 * 				VALUE=fieldValue TYPE=String|Object|ClassName>". The format
	 * 				will be:
	 * 				<Descriptor><field name="name" value="IfTable"/>
	 * 				<field name="isReadable" value="T"/></Descriptor>
	 *
     * @exception RuntimeOperationsException thrown for illegal value for field
	 * 				Names or field Values. If the XML formated string
	 * 				construction fails for any reason, this will be thrown.
     */
    public String toXMLString()
    {
		StringBuffer buff = new StringBuffer();
		buff.append("<Descriptor>");

		Object[] fieldNames = store.keySet().toArray();

		for(int i=0;i<fieldNames.length;i++)
		{
		    String field = (String)fieldNames[i];
		    if(field == null)
		       throw new RuntimeOperationsException(new RuntimeException("FieldName cannot be null"));
		    buff.append("<field name=\""+field+"\"");

		    Object fieldValue = store.get(field.toLowerCase());
		     if(fieldValue ==null)
		        throw new RuntimeOperationsException(new RuntimeException("FieldValue cannot be null"));
		    buff.append(" value=\""+fieldValue.toString()+"\" />");
		}

		buff.append("</Descriptor>");
		return buff.toString();
    }

    /**
     * A human readable string representing the descriptor. The string will be
     * in the format of "fieldName=fieldValue,fieldName2=fieldValue2,..."
     *
     * @return The string representation of descriptor
     *
     * @exception RuntimeOperationsException for illegal value for field Names
     * 				or field Values. If the descriptor string fails for any
	 * 				reason, this exception will be thrown.
     */
    public String toString()
    {
		String toRet = null;
		String[] fields = getFields();

		for(int i=0;i<fields.length;i++)
		{
			if(i == 0)
			    toRet = fields[i];
			else
			    toRet = toRet + "," + fields[i];
		}
		return toRet;
    }

	//--------------------------- Private methods ---------------------------//

	private void checkDescriptorType(String name, Object value)
											throws RuntimeOperationsException
	{
	    try
		{
            String str = (String)value;

            if(!(str.equalsIgnoreCase("attribute") ||
				 str.equalsIgnoreCase("operation") ||
               	 str.equalsIgnoreCase("notification") ||
				 str.equalsIgnoreCase("mbean") ))
                    throw new RuntimeException();
	    }
		catch(RuntimeException e)
		{
	    	throw new RuntimeOperationsException(e, "Invalid : "+name);
	    }

	}

	private void checkIsString(String name, Object value)
											throws RuntimeOperationsException
	{
	    try
		{
	        String str = (String) value;
	    }
		catch(RuntimeException e)
		{
	    	throw new RuntimeOperationsException(e, "Invalid : "+name);
	    }
	}

	private void checkIsDescriptor(String name, Object value)
											throws RuntimeOperationsException
	{
	    try
		{
	        Descriptor descr = (Descriptor)value;
	    }
		catch(RuntimeException e)
		{
	    	throw new RuntimeOperationsException(e, "Invalid : "+name);
	    }
	}

	private void checkPersistPolicy(String name, Object value)
											throws RuntimeOperationsException
	{
	    try
		{
            String str = (String)value;

            if(!(str.equalsIgnoreCase("OnUpdate") ||
				 str.equalsIgnoreCase("OnTimer") ||
               	 str.equalsIgnoreCase("NoMoreOftenThan") ||
				 str.equalsIgnoreCase("Always") ||
               	 str.equalsIgnoreCase("Never") ))
                    	throw new RuntimeException();
	    }
		catch(RuntimeException e)
		{
	    	throw new RuntimeOperationsException(e, "Invalid : "+name);
	    }
	}

	private void checkIsInt(String name, Object value)
											throws RuntimeOperationsException
	{
	    try
		{
	        int i = Integer.parseInt((String)value);
	    }
		catch(RuntimeException e)
		{
	    	throw new RuntimeOperationsException(e, "Invalid : "+name);
	    }
	}

	private void checkIsBoolean(String name, Object value)
											throws RuntimeOperationsException
	{
	    try
		{
			String str = (String)value;

			if(str != null && str.equals(""))
				value ="F";

			if( str != null && ((str.equals("true") || str.equals("false") ||
								 str.equals("T") || str.equals("F") ||
								 str.trim().equals("")) == false) )
			{
			    throw new RuntimeException();
			}
	    }
		catch(RuntimeException e)
		{
	    	throw new RuntimeOperationsException(e, "Invalid : "+name);
	    }
	}

	private void checkVisibility(String name, Object value)
											throws RuntimeOperationsException
	{
	    try
		{
			int i = Integer.parseInt((String)value);
			if(i < 1 || i > 4)
			    throw new RuntimeException();
	    }
		catch(RuntimeException e)
		{
	    	throw new RuntimeOperationsException(e, "Invalid : "+name);
	    }
	}

	private void checkRole(String name, Object value)
											throws RuntimeOperationsException
	{
	    try
		{
			String str = (String)value;
			if(!(str.equalsIgnoreCase("getter") ||
				 str.equalsIgnoreCase("setter") ||
			   	 str.equalsIgnoreCase("operation") ||
				 str.equalsIgnoreCase("constructor")))
			        	throw new RuntimeException();
	    }
		catch(RuntimeException e)
		{
	    	throw new RuntimeOperationsException(e, "Invalid : "+name);
	    }
	}

	private void checkSeverity(String name, Object value)
											throws RuntimeOperationsException
	{
	    try
		{
			int i = Integer.parseInt((String)value);
			if(i < 1 || i > 5)
			    throw new RuntimeException();
	    }
		catch(RuntimeException e)
		{
	    	throw new RuntimeOperationsException(e, "Invalid : "+name);
	    }
	}

	private void checkTargetType(String name, Object value) throws RuntimeOperationsException
	{
	    try
		{
            String str = (String)value;
            if(!(str.equalsIgnoreCase("ObjectReference") ||
				 str.equalsIgnoreCase("EJBHandle") ||
				 str.equalsIgnoreCase("Handle") ||
               	 str.equalsIgnoreCase("IOR") ||
				 str.equalsIgnoreCase("RMIReference")))
 						throw new RuntimeException();
	    }
		catch(RuntimeException e)
		{
	    	throw new RuntimeOperationsException(e, "Invalid : "+name);
	    }
	}

	private void readObject(ObjectInputStream objectinputstream)
				        		throws IOException, ClassNotFoundException
	{
	    ObjectInputStream.GetField getfield = objectinputstream.readFields();

	    try
	    {
	        store = (HashMap)getfield.get("descriptor", "");
	        currClass = (String)getfield.get("currClass",null);
	    }
	    catch(Exception exception)
	    {
	    }
	}

	private void writeObject(ObjectOutputStream objectoutputstream)
														throws IOException
	{
		ObjectOutputStream.PutField putfield = objectoutputstream.putFields();
		putfield.put("descriptor", store);
		putfield.put("currClass", currClass);

		objectoutputstream.writeFields();
	}
}