/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.modelmbean;

import java.lang.reflect.Constructor;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.ObjectStreamField;
import java.io.IOException;

import javax.management.Descriptor;
import javax.management.DescriptorAccess;
import javax.management.MBeanConstructorInfo;
import javax.management.MBeanParameterInfo;
import javax.management.IntrospectionException;

/**
 * The ModelMBeanConstructorInfo object describes a constructor of the
 * ModelMBean. It is a subclass of MBeanConstructorInfo with the addition
 * of an associated Descriptor and an implementation of the DescriptorAccess
 * interface.
 * <p>
 * <p>
 * <PRE>
 * The fields in the descriptor are defined, but not limited to, the following: <P>
 *	name           : operation name
 *	descriptorType : must be "operation"
 *	role           : must be "constructor"
 *	displayName    : human readable name of constructor
 *	class          : class where method is defined (fully qualified)
 *	visibility            : 1-4 where 1: always visible 4: rarely visible
 *	presentationString :  xml formatted string to describe how to present operation
 *</PRE>
 * <p>PersistencePolicy and CurrencyTimeLimit fields are not valid for the
 * constructor. The default constructo will have the name, descriptorType,
 * and role fields.
 */
public class ModelMBeanConstructorInfo extends MBeanConstructorInfo
									implements DescriptorAccess, Cloneable
{
	Descriptor descriptor = new DescriptorSupport();

	private static final long serialVersionUID =2675397548836767216L;

	private static final ObjectStreamField[] serialPersistentFields =
		{
			new ObjectStreamField("consDescriptor", DescriptorSupport.class),
			new ObjectStreamField("currClass",java.lang.String.class)
		};

	/**
	 * Constructs a MBeanConstructorInfo object with a default descriptor.
	 *
	 * @param description The description of the constructor
	 *
	 * @param constructor object describing the MBean constructor.
	 */
	public ModelMBeanConstructorInfo(String description, Constructor constructor)
	{
		super(description, constructor);
	}

	/**
	 * Constructs a MBeanConstructorInfo object with the specified  descriptor.
	 *
	 * @param description The description of the constructor
	 *
	 * @param constructor object describing the MBean constructor.
	 *
	 * @param descriptor  An instance of Descriptor containing the appropriate
	 * 				metadata for this instance of the ModelMBeanConstructorInfo.
	 * 				If it is null or invalid then a default desriptor will be created.
	 */
	public ModelMBeanConstructorInfo(String description,
								Constructor constructor,
								Descriptor descriptor)
	{
		super(description, constructor);
		this.descriptor = descriptor;
	}

	/**
	 * Constructs a MBeanConstructorInfo object with a default descriptor.
	 *
	 * @param name The name of the constructor
	 *
	 * @param description The description of the constructor
	 *
	 * @param signature  MBeanParameterInfo object array describing the
	 * 				parameters of the constructor.
	 */
	public ModelMBeanConstructorInfo(String name,
								 String description,
								 MBeanParameterInfo[] signature)
	{
		super(name, description, signature);
	}

    /**
     * Constructs a MBeanConstructorInfo object with a default descriptor.
     *
     * @param name The name of the constructor
     *
     * @param description The description of the constructor
     *
     * @param signature MBeanParameterInfo object array describing the
	 * 				parameters of the constructor.
	 *
	 * @param descriptor An instance of Descriptor containing the appropriate
     * 				metadata for this instance of the ModelMBeanConstructorInfo.
	 * 				If it is null or invalid then a default desriptor will be created.
	 */
	public ModelMBeanConstructorInfo(String name,
								 String description,
								 MBeanParameterInfo[] signature,
								 Descriptor descriptor)
	{
		super(name, description, signature);
		this.descriptor = descriptor;
	}

    /**
	 * Constructs a duplicate copy ModelMBeanConstructorInfo.
	 * @return The duplicate copy of the object is returned.
	 */
    public Object clone()
	{
		Descriptor clonedDescr = null;

		if(descriptor != null)
			clonedDescr = (Descriptor)descriptor.clone();

		return new ModelMBeanConstructorInfo(getName(), getDescription(),
											 getSignature(), clonedDescr);
	}

	/**
	 * Returns the descriptor
	 *
	 * @return Descriptor associated with the ModelMBeanConstructorInfo object.
	 */
	public Descriptor getDescriptor()
	{
		return descriptor;
	}

	/**
	 * Sets associated Descriptor of ModelMBeanConstructorInfo. If the new
	 * Descriptor is null, then the associated Descriptor reverts to a default
	 * descriptor. The Descriptor is validated before it is assigned. If the
	 * new Descriptor is invalid, then an IllegalArgumentException is thrown.
	 *
     * @param inDescriptor replaces the Descriptor associated with
	 * 				the ModelMBeanConstructor.
	 *
     * @throws IllegalArgumentException If the argument passsed in is illegal
	 * 				then this Exception will be thrown
	 */
    public void setDescriptor(Descriptor inDescriptor)
	{
		this.descriptor = inDescriptor;
	}

	/**
	 * Returns a human readable version of the ModelMBeanConstructorInfo instance
	 */
	public String toString()
	{
		if(descriptor != null)
			return (super.toString() + "\n Descriptor = "+ descriptor.toString());
		else
			return (super.toString() + "\n Descriptor = null");
	}

	//--------------------------- Private methods ---------------------------//

	private void readObject(ObjectInputStream objectinputstream)
						        throws IOException, ClassNotFoundException
	{
	    ObjectInputStream.GetField getfield = objectinputstream.readFields();

	    try
	    {
	        descriptor=(DescriptorSupport)getfield.get("consDescriptor", null);
	    }
	    catch(Exception exception)
	    {
	    }
	}

	private void writeObject(ObjectOutputStream objectoutputstream)
														throws IOException
	{
		ObjectOutputStream.PutField putfield = objectoutputstream.putFields();
		putfield.put("consDescriptor", descriptor);
		putfield.put("currClass", "ModelMBeanConstructorInfo");

		objectoutputstream.writeFields();
	}
}