/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.modelmbean;

import javax.management.Descriptor;
import javax.management.MBeanAttributeInfo;
import javax.management.MBeanConstructorInfo;
import javax.management.MBeanOperationInfo;
import javax.management.MBeanParameterInfo;
import javax.management.MBeanNotificationInfo;
import javax.management.MBeanException;
import javax.management.RuntimeOperationsException;

/**
* ModelMBeanInfo interface is used to expose its metadat of management attributes,
* operations and notifications for use by the management system.
* <p>
* This interface is the implemented by the ModelMBeanInfo for every ModelMBean.
* An implementation of this interface must be shipped with every JMX Agent.
* <p>
* Java resources wishing to be manageable instatiate the ModelMBean using the
* MBeanServer's createMBean method. The resource then sets the ModelMBeanInfo
* and Descriptors for the ModelMBean instance. The attributes, operations, and
* notifications exposed via the ModelMBeanInfo for the ModelMBean comprise the
* management interface and are accessible from Mbeans, connectors/adapters like
* other MBeans. Through the Descriptors, values and methods in the managed
* application can be defined and mapped to attributes and operations of the
* ModelMBean. This mapping can be defined during development in a file or
* dynamically and programmatically at runtime.
* <p>
* Every ModelMBean which is instantiated in the MBeanServer becomes manageable:
* its attributes, operations, and notifications become remotely accessible through
* the connectors/adaptors connected to that MBeanServer. A Java object cannot be
* registered in the MBeanServer unless it is a JMX compliant MBean. By instantiating
* a ModelMBean, resources are guaranteed that the MBean is valid. MBeanException and
* RuntimeOperatiosException must be thrown on every public method. This allows for
* wrappering exceptions from distributed communications (RMI, EJB, etc.)
*/
public interface ModelMBeanInfo
{
	/**
	 * To clone this ModelMBeanInfo implementation object as
	 * ModelMBeanInfoSupport object.
	 *
	 * @return The duplicate copy of the object
	 */
	public Object clone();

	/**
	 * Gets the className of the ModelMBean
	 *
	 * @return The class name in string format
	 */
	public String getClassName();

	/**
	 * Gets the description of this ModelMBean
	 *
	 * @return The description of the ModelMBean in string format
	 */
	public String getDescription();

	/**
	 * Gets the attributes info of the ModelMBean targetted object. The return
	 * object will be a array of MBeanAttributeInfo objects
	 *
	 * @return The array of MBeanAttributeInfo objects are returned after getting
	 * 				the attributes info  of the ModelMBean targetted object
	 */
	public MBeanAttributeInfo[] getAttributes();

	/**
	 * Gets the constructors info of the ModelMBean targetted object.
	 * The return object will be a array of ModelMBeanConstructorInfo objects.
	 *
	 * @return The array of MBeanConstructorInfo objects are returned after
	 * 				getting the constructors info of the ModelMBean targetted object.
	 */
	public MBeanConstructorInfo[] getConstructors();

	/**
	 * Gets the notifications info emitted by the ModelMBean targetted object.
	 * The return object will be a array of ModelMBeanNotificationInfo objects.
	 *
	 * @return The array of MBeanNotificationInfo objects are returned after
	 * 				getting the notifications info of the ModelMBean targetted object.
	 */
	public MBeanNotificationInfo[] getNotifications();

	/**
	 * Gets the operations info of the ModelMBean targetted object. The return
	 * object will be a array of ModelMBeanOperationInfo objects.
	 *
	 * @return The array of MBeanOperationInfo objects are returned after
	 * 				getting the operations info of the ModelMBean targetted object.
	 */
	public MBeanOperationInfo[] getOperations();

	/**
	 * Returns a Descriptor array consisting of all Descriptors for the
	 * ModelMBeanInfo including the MBean Descriptor, attribute Descriptors,
	 * operation Descriptors, constructor Descriptors, and notification Descriptors.
	 *
	 * @param inDescriptorType value of descriptorType field that must be set
	 * 				for the descriptor to be returned.  Must be "mbean",
	 * 				"attribute", "operation", "constructor", or "notification".
	 * 				If it is null then all types will be returned.
	 *
	 * @return Descriptor array containing all descriptors for the ModelMBean
	 *
	 * @exception MBeanException Wraps another exception
	 *
	 * @exception RuntimeOperationsException Wraps another exception
	 */
	public Descriptor[] getDescriptors(String inDescriptorType)
							throws MBeanException, RuntimeOperationsException;

	/**
	 * Adds or replaces descriptors in the ModelMBeanInfo.
	 *
	 * @param inDescriptors The descriptors to be set in the ModelMBeanInfo.
	 * 				Null elements of the list will be ignored.  All descriptors
	 * 				must have name and descriptorType fields.
	 *
	 * @exception MBeanException Wraps another exception
	 *
	 * @exception RuntimeOperationsException Wraps exceptions for illegal
	 * 				or null arguments
	 */
	public void setDescriptors(Descriptor[] inDescriptors)
							throws MBeanException, RuntimeOperationsException;

	/**
	 * Returns a Descriptor requested by name and descriptorType.
	 *
	 * @param inDescriptorName The name of the descriptor.
	 *
	 * @param inDescriptorType The type of the descriptor being requested.
	 * 				If this is null then all types are searched. Valid types
	 * 				are 'attribute', 'constructor', 'operation', and
	 * 				'notification'. This value must be equal to the
	 * 				'descriptorType' field in the descriptor that is returned.
	 *
	 * @return Descriptor containing the descriptor for the ModelMBean with the
	 * 				same name and descriptorType. If a descriptor is not found,
	 * 				null is returned.
	 *
	 * @exception MBeanException Wraps another exception
	 *
	 * @exception RuntimeOperationsException Wraps exceptions for invalid
	 * 				input name or type.
	 */
	public Descriptor getDescriptor(String inDescriptorName,
									String inDescriptorType)
							throws	MBeanException,
									RuntimeOperationsException;


	/**
	 * Adds or replaces descriptors in all the info arrays for the ModelMBean.
	 *
	 * @param inDescriptor The descriptor to be set in the ModelMBean.
	 * 				It must NOT be null. All descriptors must have name and
	 * 				descriptorType fields.
	 *
	 * @param inDescriptorType The type of the descriptor being set. If this
	 * 				is null then the descriptorType field in the descriptor
	 * 				is used. If specified this value must be set in the
	 * 				descriptorType field for the descriptor to be returned.
	 * 				Must be "mbean","attribute", "operation", "constructor",
	 * 				or "notification".
	 *
	 * @exception RuntimeOperationsException Wraps exceptions for illegal
	 * 				or null arguments.
	 */
	public void setDescriptor(Descriptor inDescriptor, String inDescriptorType)
						throws	MBeanException,
								RuntimeOperationsException;

	/**
	 * Returns a MBean's descriptor.  This descriptor contains metadata about
	 * the MBean and default policies for persistence and caching for the
	 * entire MBean. Policies may be overridden by descriptors associated with
	 * attribute, constructors, or operations with the same fieldNames.
	 *
	 * @return The MBeanDescriptor
	 *
	 * @exception MBeanException Wraps another exception
	 *
	 * @exception RuntimeOperationsException Wraps another exception
	 */
	public Descriptor getMBeanDescriptor() throws MBeanException,
												  RuntimeOperationsException;

	/**
	 * Sets the ModelMBean's descriptor (fully replace). This descriptor
	 * contains metadata about the MBean and default policies for persistence
	 * and caching for the entire MBean. Policies may be overridden by
	 * descriptors associated with attribute, constructors, or operations with
	 * the same fieldNames. This operation does a complete replacement of the
	 * descriptor, no merging is done.
	 *
	 * @param  inDescriptor This descriptor contains metadata about the MBean
	 * 				and default policies for persistence and caching for
	 * 				the entire MBean.
	 *
	 * @exception MBeanException Wraps another exception
	 *
	 * @exception RuntimeOperationsException Wraps another exception
	 */
	public void setMBeanDescriptor(Descriptor inDescriptor)
						throws MBeanException, RuntimeOperationsException;


	/**
	 * Returns a ModelMBeanAttributeInfo requested by name.
	 *
	 * @param inName The name of the ModelMBeanAttributeInfo to get.
	 *
	 * @return  ModelMBeanAttributeInfo requested by name
	 *
	 * @exception MBeanException Wraps another exception
	 *
	 * @exception RuntimeOperationsException Wraps another exception for
	 * 				invalid attribute name or ModelMBeanAttributeInfo to be returned.
	 */
	public ModelMBeanAttributeInfo getAttribute(String inName)
						throws MBeanException, RuntimeOperationsException;


	/**
	 * Returns a ModelMBeanOperationInfo requested by name.
	 *
	 * @param inName The name of the ModelMBeanOperationInfo to get.
	 *
	 * @return ModelMBeanOperationInfo requested by name
	 *
	 * @exception MBeanException Wraps another exception
	 *
	 * @exception RuntimeOperationsException Wraps another exception for
	 * 				invalid attribute name or ModelMBeanAttributeInfo to be returned.
	 */
	public ModelMBeanOperationInfo getOperation(String inName)
						throws MBeanException, RuntimeOperationsException;


	/**
	 * Returns a ModelMBeanNotificationInfo requested by name.
	 *
	 * @param inName The name of the ModelMBeanNotificationInfo to get.
	 *
	 * @return  ModelMBeanNotificationInfo requested by name
	 *
	 * @exception MBeanException Wraps another exception
	 *
	 * @exception RuntimeOperationsException Wraps another exception
	 */
	public ModelMBeanNotificationInfo getNotification(String inName)
						throws MBeanException, RuntimeOperationsException;
}