/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.modelmbean;

import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.ObjectStreamField;
import java.io.IOException;

import javax.management.Descriptor;
import javax.management.DescriptorAccess;
import javax.management.MBeanNotificationInfo;

/**
 * The ModelMBeanNotificationInfo object describes a notification emitted
 * by a ModelMBean. It is a subclass of MBeanNotificationInfo with the
 * addition of an associated Descriptor and an implementation of the
 * Descriptor interface.
 * <P>
 * <P>
 * <PRE>
 * The fields in the descriptor are defined, but not limited to, the following: <P>
 *	name           : notification name <P>
 *	descriptorType : must be "notification"   <P>
 *	severity       : 1-5 where 1: fatal 2: severe 3: error 4: warn 5: info  <P>
 *	messageID      : unique key for message text (to allow translation,analysis)  <P>
 *	messageText    : text of notification   <P>
 *	log            : T - log message F - do not log message   <P>
 *	logfile        : string fully qualified file name appropriate for operating system<P>
 *	visibility     : 1-4 where 1: always visible 4: rarely visible   <P>
 *	presentationString : xml formatted string to allow presentation of data  <P>
 * </PRE>
 * <P> The default descriptor contains the name, descriptorType, and severity=5 fields.
 */
public class ModelMBeanNotificationInfo extends MBeanNotificationInfo
									implements DescriptorAccess, Cloneable
{
	Descriptor descriptor = new DescriptorSupport();

	static final long serialVersionUID = -5304849168906325029L;

	private static final ObjectStreamField[] serialPersistentFields =
	{
		new ObjectStreamField("notificationDescriptor", DescriptorSupport.class),
		new ObjectStreamField("currClass",java.lang.String.class)
	};

	/**
	 * Constructs a ModelMBeanNotificationInfo object with a default descxriptor.
	 *
	 * @param notifTypes The notification type string
	 *
	 * @param name The Notification class name
	 *
	 * @param description The description about the notification
	 */
	public ModelMBeanNotificationInfo(String[] notifTypes,
									   String name,
									   String description)
	{
		super(notifTypes, name, description);
		this.descriptor = getDefaultDescriptor();
	}

	/**
	 * Constructs a ModelMBeanNotificationInfo object
	 *
	 * @param notifTypes The notification type string
	 *
	 * @param name The Notification class name
	 *
 	 * @param description The description about the notification
 	 *
	 * @param descriptor  Descriptor containing the appropriate metadata for
	 * 				this instance of the MBeanNotificationInfo. If it is null
	 * 				or invalid then a default desriptor will be created.
 	 */
 	public ModelMBeanNotificationInfo(String[] notifTypes,
									   String name,
									   String description,
									   Descriptor descriptor)
	{
		super(notifTypes, name, description);

		if(descriptor.isValid())
			this.descriptor = descriptor;
		else
			this.descriptor = getDefaultDescriptor();
	}

	/**
	 * Constructs a new ModelMBeanNotificationInfo object from this
	 * ModelMBeanNotfication Object.
	 *
	 * @param inInfo The duplicate ModelMBeanNotificationInfo object
	 */
	public ModelMBeanNotificationInfo(ModelMBeanNotificationInfo inInfo)
	{
		super(inInfo.getNotifTypes(), inInfo.getName(), inInfo.getDescription());

		if(inInfo.getDescriptor() != null)
			this.descriptor = (Descriptor)inInfo.getDescriptor().clone();
		else
			this.descriptor = getDefaultDescriptor();
	}

	/**
	 * Creates a duplicate ModelMBeanNotificationInfo object
	 *
	 * @return A duplicate ModelMBeanNotificationInfo object
	 */
	public Object clone()
	{
		Descriptor clonedDescr = null;

		if(descriptor != null)
			clonedDescr = (Descriptor)descriptor.clone();

		return new ModelMBeanNotificationInfo(getNotifTypes(), getName(),
											  getDescription(),clonedDescr);
	}

	/**
	 * Gets the corresponding Descriptor of this ModelMBeanNotificationInfo
	 *
	 * @return This returns the Descriptor associated with
	 * 				this ModelMBeanNotificationInfo
	 */
	public Descriptor getDescriptor()
	{
		return descriptor;
	}

	/**
	 * Sets the specified Descriptor to this ModelMBeanNotificationInfo.If the
	 * value is null then default Descriptor will be taken.  The Descriptor
	 * is validated before it is assigned. If the new Descriptor is invalid,
	 * then an IllegalArgumentException is thrown.
	 *
	 * @param inDescriptor This replaces the Descriptor associated with the
	 * 				ModelMBeanNotification interface
	 *
	 * @throws IllegalArgumentException If the new Descriptor is invalid,
	 * 				then it will be thrown.
	 */
	public void setDescriptor(Descriptor inDescriptor)
	{
		if(inDescriptor == null)
		{
            this.descriptor = getDefaultDescriptor();
		}
		else
		{
			if(inDescriptor.isValid())
	 			this.descriptor = (Descriptor) inDescriptor.clone();
	 		else
				throw new IllegalArgumentException("Invalid descriptor received as parameter");
		}
	}

	/**
	 * Returns a human readable version of the ModelMBeanNotificationInfo instance
	 * @return  Human readable version of the ModelMBeanNotificationInfo instance
	 */
	public String toString()
	{
		if(descriptor != null)
			return (super.toString() + "\n Descriptor = "+ descriptor.toString());
		else
			return (super.toString() + "\n Descriptor = null");
	}

	//--------------------------- Private methods ---------------------------//

	/**
	 * Creates default descriptor for notification as follows:
	 * descriptorType=notification, name=this.getName(),
	 * displayname=this.getName(),log=F,severity=5,visibility=1
	 */
	private Descriptor getDefaultDescriptor()
	{
		return new DescriptorSupport(
						new String[] {
								"descriptorType=notification",
								("name=" + this.getName()),
								("displayName=" + this.getName()),
								 "severity=5",
								 "log=F",
								 "visibility=1"
								 });
	}

	private void readObject(ObjectInputStream objectinputstream)
					        		throws IOException, ClassNotFoundException
	{
	    ObjectInputStream.GetField getfield = objectinputstream.readFields();

	    try
	    {
	        descriptor=(DescriptorSupport)getfield.get("notificationDescriptor", null);
	    }
	    catch(Exception exception)
	    {
	        //throw new IOException(malformedobjectnameexception.toString());
	    }
	}

	private void writeObject(ObjectOutputStream objectoutputstream)
											          throws IOException
	{
		ObjectOutputStream.PutField putfield = objectoutputstream.putFields();
		putfield.put("notificationDescriptor", descriptor);
		putfield.put("currClass", "ModelMBeanNotificationInfo");

		objectoutputstream.writeFields();
	}
}