/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.monitor;

import javax.management.ObjectName;
import javax.management.MBeanServer;
import javax.management.MBeanNotificationInfo;
import javax.management.MBeanException;
import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.ReflectionException;

/**
 * Defines a monitor MBean designed to observe the values of a gauge attribute.
 * <P>
 * A gauge monitor observes an attribute that is continuously variable with time.
 * A gauge monitor sends notifications as follows:
 * <UL>
 * <LI> if the attribute value is increasing and becomes equal to or greater
 * 		than the high threshold value,
 *      a {@link MonitorNotification#THRESHOLD_HIGH_VALUE_EXCEEDED threshold high notification}
 * 		is sent. The notify high flag must be set to <CODE>true</CODE>
 *      <BR>Subsequent crossings of the high threshold value do not cause
 * 		further notifications unless the attribute value becomes equal to or
 * 		less than the low threshold value.
 * <LI> if the attribute value is decreasing and becomes equal to or less than
 * 		the low threshold value,
 *      a {@link MonitorNotification#THRESHOLD_LOW_VALUE_EXCEEDED threshold low notification}
 * 		is sent. The notify low flag must be set to <CODE>true</CODE>.
 *      <BR>Subsequent crossings of the low threshold value do not cause
 * 		further notifications unless the attribute value becomes equal to or
 * 		greater than the high threshold value.
 * </UL>
 * This provides a hysteresis mechanism to avoid repeated triggering of
 * notifications when the attribute value makes small oscillations around the
 * high or low threshold value.
 * <P>
 * If the gauge difference mode is used, the value of the derived gauge is
 * calculated as the difference between the observed gauge values for two
 * successive observations.
 * <BR>The derived gauge value (V[t]) is calculated using the following method:
 * <UL>
 * <LI>V[t] = gauge[t] - gauge[t-GP]
 * </UL>
 * This implementation of the gauge monitor requires the observed attribute to
 * be of the type integer or floating-point (<CODE>Byte</CODE>,
 * <CODE>Integer</CODE>, <CODE>Short</CODE>, <CODE>Long</CODE>,
 * <CODE>Float</CODE>, <CODE>Double</CODE>).
 */
public class GaugeMonitor extends Monitor implements GaugeMonitorMBean
{
    /**
     * Derived gauge.
     */
	private		Number		derivedGauge			=	null;

    /**
     * Flag indicating if the gauge difference mode is used.
     * If the gauge difference mode is used, the derived gauge is the difference between two consecutive observed values.
     * Otherwise, the derived gauge is directly the value of the observed attribute.
     * <BR>The default value is set to <CODE>false</CODE>.
     */
	private		boolean		differenceMode			=	false;

    /**
     * Gauge high threshold.
     * <BR>The default value is a null Integer object.
     */
	private		Number		highThreshold			=	null;

    /**
     * Gauge low threshold.
     * <BR>The default value is a null Integer object.
     */
	private		Number		lowThreshold			=	null;

    /**
     * Flag indicating if the gauge monitor notifies when exceeding the high threshold.
     * <BR>The default value is set to <CODE>false</CODE>.
     */
	private		boolean		notifyHigh				=	true;

    /**
     * Flag indicating if the gauge monitor notifies when exceeding the low threshold.
     * <BR>The default value is set to <CODE>false</CODE>.
     */
	private		boolean		notifyLow				=	true;

    /**
     * Previous derived gauge value.
     */
	private		Number		prevDerivedGauge		=	null;

	private	long sequenceNumber						=	1;
	private	Comparable comparableDerivedGauge		=	null;
	private	String observedAttributeType			=	null;
	private	Comparable comparableLowThreshold 		=	null;
	private	Comparable comparableHighThreshold 		=	null;
	private ObjectName objName						= 	null;
	private	Thread gaugeMonitorThread				=	null;
	private int NO_NOTIF_SENT 						= 	1;
	private int HIGH_NOTIF_SENT 					= 	2;
	private int LOW_NOTIF_SENT 						= 	3;
	private int notifStatus;


	/**
	 * Default Constructor.
	 */
	public GaugeMonitor()
	{
		try
		{
			objName = new ObjectName("Services:type=GaugeMonitor");
		}
		catch(Exception e)
		{
		}
		//notifStatus = NO_NOTIF_SENT;
	}

	/**
	 * Starts the Gauge monitor.
	 */
	public void start()
	{
		Object object = null;
		gaugeMonitorThread = new Thread(new GaugeMonitorThread(this));
		isActive = true;
		notifStatus = NO_NOTIF_SENT;
		gaugeMonitorThread.setName("GaugeMonitorThread-"+gaugeMonitorThread.getName());
		gaugeMonitorThread.start();
	}

	/**
	 * Stops the Gauge Monitor.
	 */
	public void stop()
	{
		isActive = false;
		if(gaugeMonitorThread != null)
		{
			if(gaugeMonitorThread.isAlive())
			{
				gaugeMonitorThread.stop();
			}
		}
	}

	/**
	 * This method returns a NotificationInfo object containing the name of the
	 * Java class of the notification and the notification types sent by the
	 * counter monitor.
	 *
	 * @return An Array of MBeanNotificationInfo objects.
	 */
	public MBeanNotificationInfo[] getNotificationInfo()
	{
		//Check if this returns the correct values---------balaji.
		MBeanNotificationInfo[] notifInfo = super.getNotificationInfo();

		MBeanNotificationInfo[] newNotifInfo = new MBeanNotificationInfo[notifInfo.length+2];

		for(int i=0;i<notifInfo.length;i++)
		{
			newNotifInfo[i] = notifInfo[i];
		}

		String[] types = {"jmx.monitor.error.threshold"};
		newNotifInfo[newNotifInfo.length - 2] = new MBeanNotificationInfo(types,"Monitor Threshold Error Notification","Emimitted for a error case in Threshold");

		String[] types1 = {"jmx.monitor.gauge.high","jmx.monitor.gauge.low"};
		newNotifInfo[newNotifInfo.length - 1] = new MBeanNotificationInfo(types1,"Monitor Threshold CrossOver Notification","Emimitted for a CrossOver in monitored attribute");

		return newNotifInfo;
	}

	/**
	 * This method gets the value of the derived gauge. The derived gauge is
	 * either the exact value of the observed attribute, or the difference
	 * between the two consecutive observed values of the attribute.
	 *
	 * @return An instance of java.lang.Number giving the value of the derived gauge.
	 */
	public java.lang.Number getDerivedGauge()
	{
		return derivedGauge;
	}

	/**
	 * This method gets the value of the derived gauge time stamp.The derived
	 * gauge time stamp* is the value(in the nearest miliseconds) when the
	 * notification was triggered.
	 *
	 * @return long value representing the time the notification was triggered.
	 */
	public long getDerivedGaugeTimeStamp()
	{
		return derivedGaugeTimeStamp;
	}

	/**
	 * This method gets the value of the difference mode. If the difference
	 * mode is true, the difference mode option is set to calculate the value
	 * of the derived gauge.
	 *
	 * @return boolean value indicating whether the difference mode option is set.
	 */
	public boolean getDifferenceMode()
	{
		return differenceMode;
	}

	/**
	 * This method sets the state of the difference mode.
	 *
	 * @param value boolean value representing the state of the difference mode.
	 */
	public void setDifferenceMode(boolean value)
	{
		this.differenceMode = value;
	}

	/**
	 * This method gets the value of the high threshold.
	 *
	 * @return An instance of java.lang.Number giving the value of the high threshold.
	 */
	public java.lang.Number getHighThreshold()
	{
		return highThreshold;
	}

	/**
	 * This method gets the value of the low threshold.
	 *
	 * @return An instance of java.lang.Number giving the value of the low threshold.
	 */
	public java.lang.Number getLowThreshold()
	{
		return lowThreshold;
	}

	/**
	 * This method sets the high threshold and low threshold values.
	 *
	 * @param	highValue - The high threshold value.
	 *
	 * @param	lowValue - The low threshold value.
	 *
	 * @exception java.lang.IllegalArgumentException - The specified high/low
	 * 				threshold is null or the low threshold is greater than the
	 * 				high threshold or the high threshold and the low threshold
	 * 				are not of the same type.
	 */
	public void setThresholds(Number highValue, Number lowValue)
										throws IllegalArgumentException
	{
		Object object = null;

		if(highValue == null || lowValue == null)
		{
			throw new IllegalArgumentException("Threshold Values cannot be null");
		}

		String highValueType = highValue.getClass().getName();
		String lowValueType = lowValue.getClass().getName();

		//if the highValue and lowValue are not of the same type,throw exception.
		this.highThreshold = highValue;
		this.lowThreshold  = lowValue;

		if(!( highValueType.equals(lowValueType)))
		{
			sendNotification(new MonitorNotification(MonitorNotification.THRESHOLD_ERROR,this,												 observedObject,attributeName,derivedGauge,													 highValue,sequenceNumber++));
			throw new IllegalArgumentException("Threshold values should be of the same type");
		}

		if(getComparableValue(highValue).compareTo(getComparableValue(lowValue)) < 0)
		{
			/*sendNotification(new MonitorNotification(MonitorNotification.THRESHOLD_ERROR,this,													 observedObject,attributeName,derivedGauge,													 lowValue,sequenceNumber++));
			*/
			throw new IllegalArgumentException("Low Threshold cannot be greater than High Threshold");
		}

		//TODO --- if the HighThresholdDataType && LowThresholdDataType are not
		//the same as ObservedAttribute DataType ....send a notificiaiton.

		//gets the instances of the thresholds as instances of Comparable interface.
		comparableLowThreshold = getComparableValue(lowThreshold);
		comparableHighThreshold = getComparableValue(highThreshold);
	}

	/**
	 * This method gets the high notification's on/off switch value.
	 *
	 * @return true if the gauge monitor notifies when exceeding the high
	 * 				threshold, false otherwise.
	 */
	public boolean getNotifyHigh()
	{
		return notifyHigh;
	}

	/**
	 * This method sets the high notification's on/off switch value.
	 *
	 * @param value - The high notification's on/off switch value.
	 */
	public void setNotifyHigh(boolean value)
	{
		this.notifyHigh = value;
	}

	/**
	 * This method gets the low notification's on/off switch value.
	 *
	 * @return true if the gauge monitor notifies when exceeding the low
	 * 				threshold, false otherwise.
	 */
	public boolean getNotifyLow()
	{
		return notifyLow;
	}

	/**
	 * This method sets the low notification's on/off switch value.
	 *
	 * @param value - The low notification's on/off switch value.
	 */
	public void setNotifyLow(boolean value)
	{
		this.notifyLow = value;
	}

	/**
	 * Overriding the preRegister() of MBeanRegistration interface to store the
	 * ObjectName locally.
	 */
	public ObjectName preRegister(MBeanServer server, ObjectName name)
														throws Exception
	{
		this.objName = name;
		super.preRegister(server,name);
		return name;
	}

	//-------------------------- Private methods --------------------------//

	// Calculates the derived gauge according to the specifications when the
	// difference mode is set.
	private Number calculateDerivedGauge(Object derived, Object prevDerived)
	{
		Number toRet = null;
		String temp = derived.getClass().getName();

		if(temp.equals("java.lang.Integer")){
			toRet = (Number)( new Integer(((Integer)derived).intValue() -
										  ((Integer)prevDerived).intValue()));
		}
		if(temp.equals("java.lang.Float")){
			toRet = (Number)( new Float(((Float)derived).floatValue() -
										  ((Float)prevDerived).floatValue()));
		}
		if(temp.equals("java.lang.Double")){
			toRet = (Number)( new Double(((Double)derived).doubleValue() -
										  ((Double)prevDerived).doubleValue()));
		}
		if(temp.equals("java.lang.Long")){
			toRet = (Number)( new Long(((Long)derived).longValue() -
										  ((Long)prevDerived).longValue()));
		}

		return toRet;
	}

	//checks whether the observed attribute is only of gauge monitor type.
	private boolean isGaugeMonitorDataType(Object object)
	{
		String temp = object.getClass().getName();

		if(temp.indexOf("Integer") != -1)
			return true;
		else if(temp.indexOf("Float") != -1 )
			return true;
		else if(temp.indexOf("Double") != -1 )
			return true;
		else if(temp.indexOf("Long") != -1 )
			return true;
		else
			return false;
	}

	//This method checks whether the observed atttribute is of a gauge Monitor Type.
	private Comparable getComparableValue(Object object)
	{
		Comparable toRet = null;
		String temp = object.getClass().getName();

		if(temp.indexOf("Integer") != -1)
			toRet =  (Integer)object;
		else if(temp.indexOf("Float") != -1 )
			toRet =  (Float)object;
		else if(temp.indexOf("Double") != -1)
			toRet =  (Double)object;
		else if(temp.indexOf("Long") != -1)
			toRet =  (Long)object;
		return toRet;
	}


	//---------------------------- Inner class -----------------------------//

	// This inner class implements Runnable and when the StringMonitor is
	// started, the start method of this class is called and it periodically
	// polls the Observed Attribute according to the granularity period and
	// when there is a deviation from the configuration details, sends a notification.
	class GaugeMonitorThread implements Runnable
	{
		Object obj = null;
		MonitorNotification notif = null;
		GaugeMonitor monitor = null;
		private boolean isNotifHighSent = false;
		private boolean isNotifLowSent  = false;

		GaugeMonitorThread(GaugeMonitor monitor)
		{
			this.monitor = monitor;
		}

		public void run()
		{
			while(true)
			{
				try
				{
					obj = server.getAttribute(observedObject,attributeName);
					derivedGaugeTimeStamp = System.currentTimeMillis();
				}
				catch(InstanceNotFoundException ine)
				{
					if(Monitor.OBSERVED_OBJECT_ERROR_NOTIFIED == 1)
					{
						sendNotification(new MonitorNotification(
								MonitorNotification.OBSERVED_OBJECT_ERROR,
								monitor, observedObject, attributeName,
								null, null, sequenceNumber++));
						Monitor.OBSERVED_OBJECT_ERROR_NOTIFIED = 0;
					}
					sleep();
					continue;
				}
				catch(AttributeNotFoundException ane)
				{
					 if(Monitor.OBSERVED_ATTRIBUTE_ERROR_NOTIFIED == 1)
					 {
						 sendNotification(new MonitorNotification(
						 		MonitorNotification.OBSERVED_ATTRIBUTE_ERROR,
								monitor, observedObject, attributeName,
								null, null, sequenceNumber++));
						 Monitor.OBSERVED_ATTRIBUTE_ERROR_NOTIFIED = 0;
					 }
					 sleep();
					 continue;
				}
				catch(MBeanException mbe)
				{
					if(Monitor.RUNTIME_ERROR_NOTIFIED  == 1)
					{
						sendNotification(new MonitorNotification(
								MonitorNotification.RUNTIME_ERROR,
								monitor, observedObject, attributeName,
								null, null, sequenceNumber++));
						Monitor.RUNTIME_ERROR_NOTIFIED = 0;
					}
					sleep();
					continue;
				}
				catch(ReflectionException re)
				{
					 if(Monitor.OBSERVED_ATTRIBUTE_ERROR_NOTIFIED == 1)
					 {
						 sendNotification(new MonitorNotification(
						 		MonitorNotification.OBSERVED_ATTRIBUTE_ERROR,
								monitor, observedObject, attributeName,
								null, null, sequenceNumber++));
						 Monitor.OBSERVED_ATTRIBUTE_ERROR_NOTIFIED = 0;
					 }
					 sleep();
					 continue;
				}
				catch(Exception e)
				{
					 if(Monitor.RUNTIME_ERROR_NOTIFIED == 1)
					 {
						 sendNotification(new MonitorNotification(
						 		MonitorNotification.RUNTIME_ERROR,
								monitor, observedObject, attributeName,
								null, null, sequenceNumber++));
						 Monitor.RUNTIME_ERROR_NOTIFIED = 0;
					 }
					 sleep();
					 continue;
				}

				if(obj == null)
				{
					continue;
				}

				//check if the attribute is of gaugemonitor type.
				if( ! isGaugeMonitorDataType(obj))
				{
					if(Monitor.OBSERVED_ATTRIBUTE_TYPE_ERROR_NOTIFIED  == 1)
					{
						sendNotification(new MonitorNotification(
								MonitorNotification.OBSERVED_ATTRIBUTE_TYPE_ERROR,
								monitor, observedObject, attributeName,
								null, null, sequenceNumber++));
						Monitor.OBSERVED_ATTRIBUTE_TYPE_ERROR_NOTIFIED = 0;
					}
				}

				derivedGauge = (Number)obj;

				comparableDerivedGauge = getComparableValue(derivedGauge);

				if(differenceMode)
				{
					// calculate the derivedGauge Value for the difference Mode.
					if(prevDerivedGauge == null)
					{
						prevDerivedGauge = derivedGauge;
					}

					derivedGauge = calculateDerivedGauge(derivedGauge,prevDerivedGauge);
					comparableDerivedGauge = getComparableValue(derivedGauge);
				}

				/*if(comparableDerivedGauge.compareTo(comparableHighThreshold) >= 0)
				{
					if(notifyHigh)
					{
						notif = new MonitorNotification(MonitorNotification.THRESHOLD_HIGH_VALUE_EXCEEDED,monitor,														observedObject,attributeName,														derivedGauge,highThreshold,														sequenceNumber++);
						sendNotification(notif);

						notifyHigh = false;
						notifyLow = true;
					}
				}

				if(comparableDerivedGauge.compareTo(comparableLowThreshold) <= 0)
				{
					if(notifyLow)
					{
						notif = new MonitorNotification(MonitorNotification.THRESHOLD_LOW_VALUE_EXCEEDED,monitor,														observedObject,attributeName,														derivedGauge,lowThreshold,														sequenceNumber++);
						sendNotification(notif);

						notifyLow = false;
						notifyHigh = true;
					}
				}*/

				if(notifyHigh)
				{
					if(isNotifHighSent)
					{
						if(comparableDerivedGauge.compareTo(comparableHighThreshold) < 0)
						{
							isNotifHighSent = false;
							if(!notifyLow)
								notifStatus = LOW_NOTIF_SENT;
						}
					}

					if(notifStatus == NO_NOTIF_SENT || notifStatus == LOW_NOTIF_SENT)
					{
						/*
						if(isNotifHighSent){
							if(comparableDerivedGauge.compareTo(comparableHighThreshold) < 0){
								isNotifHighSent = false;
								if(!notifyLow)
									notifStatus = LOW_NOTIF_SENT;
							}
						}
						else{
						*/
						if(comparableDerivedGauge.compareTo(comparableHighThreshold) >=0)
						{
							notif = new MonitorNotification(
								MonitorNotification.THRESHOLD_HIGH_VALUE_EXCEEDED,
								monitor, observedObject, attributeName,
								derivedGauge, highThreshold, sequenceNumber++);
							isNotifHighSent = true;

							sendNotification(notif);
							notifStatus = HIGH_NOTIF_SENT;
						}
						//}
					}
				}

				if(notifyLow)
				{
					if(isNotifLowSent)
					{
						if(comparableDerivedGauge.compareTo(comparableLowThreshold) >0)
						{
							isNotifLowSent = false;
							if(!notifyHigh)
								notifStatus = HIGH_NOTIF_SENT;
						}
					}

					if(notifStatus == NO_NOTIF_SENT || notifStatus == HIGH_NOTIF_SENT)
					{
						/*
						if(isNotifLowSent){
							if(comparableDerivedGauge.compareTo(comparableLowThreshold) >0){
								isNotifLowSent = false;
								if(!notifyHigh)
									notifStatus = HIGH_NOTIF_SENT;
							}
						}
						else {
						*/

						if(comparableDerivedGauge.compareTo(comparableLowThreshold) <=0)
						{
							notif = new MonitorNotification(
								MonitorNotification.THRESHOLD_LOW_VALUE_EXCEEDED,
								monitor, observedObject, attributeName,
								derivedGauge, highThreshold, sequenceNumber++);
							isNotifLowSent = true;
							sendNotification(notif);
							notifStatus = LOW_NOTIF_SENT;
						}
						//}
					}
				}

				try
				{
					prevDerivedGauge = derivedGauge;
					Thread.sleep(granularityPeriod);

				}
				catch(InterruptedException IE)
				{
					//IE.printStackTrace();
				}
			}
		}

		private void sleep()
		{
			try
			{
				Thread.sleep(granularityPeriod);
			}
			catch(Exception e)
			{
			}
		}
	}//End GaugeMonitor Thread class.
}//End GaugeMonitor Class.