/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.relation;

import java.util.List;
import java.util.ArrayList;

import javax.management.ObjectName;
import javax.management.Notification;

/**
 * A RelationNotification notification is sent when a relation is created via
 * the Relation Service, or a MBean is added as a relation in the Relation
 * Service, or a role is updated in a relation, or a relation is removed
 * from the Relation Service
 */
public class RelationNotification extends Notification
{
	/**
	 * Type for the creation of an internal relation
	 */
	public static final String RELATION_BASIC_CREATION = "jmx.relation.creation.basic";

	/**
	 * Type for the removal from the Relation Service of an internal relation
	 */
	public static final String RELATION_BASIC_REMOVAL = "jmx.relation.removal.basic";

	/**
	 * Type for an update of an internal relation
	 */
	public static final String RELATION_BASIC_UPDATE = "jmx.relation.update.basic";

	/**
	 * Type for the relation MBean added into the Relation Service
	 */
	public static final String RELATION_MBEAN_CREATION = "jmx.relation.creation.mbean";

	/**
	 * Type for the removal from the Relation Service of a relation MBean
	 */
	public static final String RELATION_MBEAN_REMOVAL = "jmx.relation.removal.mbean";

    /**
	 * Type for the update of a relation MBean
	 */
	public static final String RELATION_MBEAN_UPDATE = "jmx.relation.update.mbean";

	String type = null;
	Object source = null;
	long seqno = 0;
	long timeStamp = 0;

    // Relation identifier of created/removed/updated relation
	String relId = null;

    // Relation type name of created/removed/updated relation
	String relTypeName = null;

	String message = null;

    // ObjectName of the relation MBean of created/removed/updated relation
    // (only if the relation is represented by a MBean)
	ObjectName relObjectName = null;

    // List of MBean ObjectNames to be unregistered
	ArrayList unRegMBeanList = null;

    // Name of updated role (only for role update)
	String roleName = null;

	ArrayList newRoleValue = new ArrayList();

    // Old role value (ArrayList of ObjectNames) (only for role update)
	ArrayList oldRoleValue = new ArrayList();


	/**
	 * Creates a notification for either a relation creation (RelationSupport
	 * object created internally in the Relation Service, or a MBean added as
	 * a relation) or for a relation removal from the Relation Service
	 *
	 * @param theNtfType - type of the notification; either:
	 *           - RELATION_BASIC_CREATION
	 *           - RELATION_MBEAN_CREATION
	 *           - RELATION_BASIC_REMOVAL
	 *           - RELATION_MBEAN_REMOVAL
	 *
	 * @param  theSrcObj - source object, sending the notification. Will always
	 * 				be a RelationService object.
	 *
	 * @param theSeqNbr - sequence number to identify the notification
	 *
	 * @param theTimeStamp - time stamp
	 *
	 * @param theMsg - human-readable message describing the notification
	 *
	 * @param theRelId - relation id identifying the relation in the Relation Service
	 *
	 * @param theRelTypeName - name of the relation type
	 *
	 * @param theRelObjName - ObjectName of the relation object if it is a MBean
	 * 				(null for relations internally handled by the Relation Service)
	 *
	 * @param theUnregMBeanList - list of ObjectNames of referenced MBeans
	 * 				expected to be unregistered due to relation removal (only
	 * 				for removal, due to CIM qualifiers, can be null)
	 *
	 * @throws java.lang.IllegalArgumentException - if:
	 * 				- no value for the notification type
	 * 				- the notification type is not RELATION_BASIC_CREATION,
	 * 					RELATION_MBEAN_CREATION, RELATION_BASIC_REMOVAL
	 * 					or RELATION_MBEAN_REMOVAL
	 * 				- no source object
	 * 				- the source object is not a Relation Service
	 * 				- no relation id
	 * 				- no relation type name
	 */
	public RelationNotification(String theNtfType,
		                        Object theSrcObj,
			                    long theSeqNbr,
				                long theTimeStamp,
					            String theMsg,
						        String theRelId,
							    String theRelTypeName,
								ObjectName theRelObjName,
								List theUnregMBeanList)
					throws IllegalArgumentException
	{
		super(theNtfType,theSrcObj,theSeqNbr);

		if(theNtfType == null || theNtfType.equals(""))
		   throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);

		if( !(this.isCreateOrRemovalNotificationType(theNtfType)) )
		    throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);

		if(theSrcObj == null )
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);

		if(theRelId ==null || theRelTypeName == null)
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);

		this.type = theNtfType;
		this.source = theSrcObj;
		this.seqno = theSeqNbr;
		this.timeStamp = theTimeStamp;
		this.message = theMsg;
		this.relId = theRelId;
		this.relObjectName = theRelObjName;
		this.relTypeName = theRelTypeName;
		this.unRegMBeanList = (ArrayList)theUnregMBeanList;
	}

	/**
	 * Creates a notification for a role update in a relation
	 *
	 * @param theNtfType - type of the notification; either:
	 * 				- RELATION_BASIC_UPDATE
	 * 				- RELATION_MBEAN_UPDATE
	 *
	 * @param theSrcObj - source object, sending the notification. Will always
	 * 				be a RelationService object.
	 *
	 * @param theSeqNbr - sequence number to identify the notification
	 *
	 * @param theTimeStamp - time stamp
	 *
	 * @param theMsg - human-readable message describing the notification
	 *
	 * @param theRelId - relation id identifying the relation in the Relation Service
	 *
	 * @param theRelTypeName - name of the relation type
	 *
	 * @param theRelObjName - ObjectName of the relation object if it is a MBean
	 * 				(null for relations internally handled by the Relation Service)
	 *
	 * @param theRoleName - name of the updated role
	 *
	 * @param theNewRoleValue - new value (ArrayList of ObjectName objects)
	 *
	 * @param theOldRoleValue - old value (ArrayList of ObjectName objects)
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 */
	public RelationNotification(String theNtfType,
	                            Object theSrcObj,
		                        long theSeqNbr,
			                    long theTimeStamp,
				                String theMsg,
					            String theRelId,
						        String theRelTypeName,
							    ObjectName theRelObjName,
								String theRoleName,
	                            List theNewRoleValue,
				                List theOldRoleValue)
					throws IllegalArgumentException
	{
		super(theNtfType,theSrcObj,theSeqNbr);

		if(theNtfType == null || theNtfType.equals(""))
		   throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);

		if( !(this.isUpdateNotificationType(theNtfType)) )
		    throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);

		if(theSrcObj == null )
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);

		if(theRelId ==null || theRelTypeName == null)
			throw new IllegalArgumentException(RelationErrs.ERR_ILLEGAL_ARG);

		this.type = theNtfType;
		this.source = theSrcObj;
		this.seqno = theSeqNbr;
		this.timeStamp = theTimeStamp;
		this.message = theMsg;
		this.relId = theRelId;
		this.relObjectName = theRelObjName;
		this.relTypeName = theRelTypeName;
		this.roleName = theRoleName;
		this.oldRoleValue = (ArrayList)theOldRoleValue;
		this.newRoleValue = (ArrayList)theNewRoleValue;
	}

	/**
	 * Returns the list of ObjectNames of MBeans expected to be unregistered
	 * due to a relation removal (only for relation removal)
	 *
	 * @return The list of ObjectNames of MBeans expected to be unregistered
	 * 				due to a relation removal
	 */
	public List getMBeansToUnregister()
	{
        return this.unRegMBeanList;
	}

	/**
	 * Returns new value of updated role (only for role update)
	 *
	 * @return New role value
	 */
	public List getNewRoleValue()
	{
		return this.newRoleValue;
	}

	/**
	 * Returns the ObjectName (if the relation is a MBean, else null) of the
	 * created/removed/updated relation
	 *
	 * @return  Name of the Object
	 */
	public ObjectName getObjectName()
	{
		return this.relObjectName;
	}

	/**
	 * Returns old value of updated role (only for role update)
	 *
	 * @return The old role value
	 */
	public List getOldRoleValue()
	{
		return this.oldRoleValue;
	}

	/**
	 * Returns the relation identifier of created/removed/updated relation
	 *
	 * @return The relation identifier of created/removed/updated relation
	 */
	public String getRelationId()
	{
		return this.relId;
	}

	/**
	 * Returns the relation type name of created/removed/updated relation
	 *
	 * @return The relation type name of created/removed/updated relation
	 */
	public String getRelationTypeName()
	{
		return this.relTypeName;
	}

	/**
	 * Returns name of updated role of updated relation (only for role update)
	 *
	 * @return The name of the updated role of updated relation
	 */
	public String getRoleName()
	{
		return this.roleName;
	}


    //*************************** private Methods ***************************//

	private boolean isCreateOrRemovalNotificationType(String notifType)
	{
	    if(notifType.equals(RelationNotification.RELATION_BASIC_CREATION))
	        return true;
	    else if(notifType.equals(RelationNotification.RELATION_MBEAN_CREATION))
	        return true;
	    else if(notifType.equals(RelationNotification.RELATION_BASIC_REMOVAL))
	        return true;
        else if(notifType.equals(RelationNotification.RELATION_MBEAN_REMOVAL))
	        return true;
	    else
	        return false;
	}

	private boolean isUpdateNotificationType(String notifType)
	{
	    if(notifType.equals(RelationNotification.RELATION_BASIC_UPDATE))
	        return true;
	    else if(notifType.equals(RelationNotification.RELATION_MBEAN_UPDATE))
	        return true;
	    else
    	    return false;
	}
}