/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.relation;

import java.io.Serializable;
import java.util.List;
import java.util.Iterator;

import javax.management.ObjectName;

/**
 * Represents a role: includes a role name and referenced MBeans (via their
 * ObjectNames). The role value is always represented as an ArrayList
 * collection (of ObjectNames) to homogenise the access.
 */
public class Role implements Serializable
{
	List valueList = null;
	String name = null;

	/**
	 * Constructor.  No verification about the provided ObjectNames, no need
	 * for them to be registered at this level. This checking will be made
	 * when trying to set a relation role.
	 *
	 * @param theRoleName - role name
	 *
	 * @param theRoleValue - role value (ArrayList of ObjectName objects)
	 *
	 * @exception java.lang.IllegalArgumentException - if null parameter
	 */
	public Role(String theRoleName, List theRoleValue)
											throws IllegalArgumentException
	{
		if( theRoleName == null || theRoleValue == null)
			throw new IllegalArgumentException(
							"Cannot create a Role Object with null values");

		this.name = theRoleName;
		this.valueList = theRoleValue;
	}

	/**
	 * Cloning.Creates a duplicate copy for the current Role object
	 *
	 * Overrides:
	 *     clone in class java.lang.Object
	 *
	 * @return
	 *    a Role being an independent copy of the current Role object.
	 */
	public Object clone()
	{
    	return new Role(getRoleName(),getRoleValue());
	}

	/**
	 * Retrieves role name
	 *
	 * @return This gets the name of role
	 */
	public String getRoleName()
	{
		return this.name;
	}

	/**
	 * Retrieves role value
	 *
	 * @return
	 *   ArrayList of ObjectName objects for referenced MBeans.
	 */
	public List getRoleValue()
	{
		return this.valueList;
	}

	/**
	 * Returns a string for the given role value
	 *
	 * @param theRoleValue - ArrayList of ObjectName objects
	 *
	 * @return String being the ObjectNames separated by commas.
	 *
	 * @exception java.lang.IllegalArgumentException - if null parameter
	 */
	public static String roleValueToString(List theRoleValue)
										throws IllegalArgumentException
	{
		if(theRoleValue == null)
			throw new IllegalArgumentException("Cannot convert null RoleValue Objects to String ");

		//return the Appended form of all the ObjectNames in the List.
		StringBuffer  str = null;
		Iterator i = theRoleValue.iterator();
		while(i.hasNext())
		{
			ObjectName temp = (ObjectName)(i.next());
			str.append(temp.toString());
			if(i.hasNext())
				str.append(",");
		}
		String toRet = str.toString();

		return toRet.substring(0,toRet.length() - 1) ;
	}

	/**
	 * Sets role name
	 *
	 * @param theRoleName - role name
	 *
	 * @exception java.lang.IllegalArgumentException - if null parameter
	 */
	public void setRoleName(String theRoleName) throws IllegalArgumentException
	{
		if(theRoleName == null)
			throw new IllegalArgumentException("Cannot set null Objects for the RoleNames");

		this.name = theRoleName;
	}

	/**
	 * Sets role value
	 *
	 * @param theRoleValue - ArrayList of ObjectName objects for referenced MBeans.
	 *
	 * @exception java.lang.IllegalArgumentException - if null parameter
	 */
	public void setRoleValue(List theRoleValue) throws IllegalArgumentException
	{
		if(theRoleValue == null)
			throw new IllegalArgumentException();

		this.valueList = theRoleValue;
	}

	/**
	 * Prints a string describing the role
	 *
	 * Overrides: toString in class java.lang.Object
	 *
	 * @return String format of the role
	 */
	public String toString()
	{
        StringBuffer str = new StringBuffer();
        str.append(getRoleName());
        str.append(":");

        //return the Appended form of all the ObjectNames in the List.
        for(int i=0 ; i<valueList.size(); i++)
		{
            str.append(((ObjectName)(valueList.get(i))).toString());
            str.append(",");
        }
        String toRet = str.toString();

        return toRet.substring(0,str.length() -1 );
	}
}