/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.relation;

import java.lang.reflect.Constructor;
import java.lang.reflect.Modifier;
import java.io.Serializable;

import javax.management.MBeanInfo;
import javax.management.MBeanConstructorInfo;
import javax.management.NotCompliantMBeanException;

import com.adventnet.jmx.DefaultDynamicMBean;

/**
 * A RoleInfo object represents a role information in a relation type.
 */
public class RoleInfo implements Serializable
{
	/**
	 * To specify an unlimited cardinality
	 */
	public static int ROLE_CARDINALITY_INFINITY = 0;


    // Name of the role
	String roleName = null;

    // Name of class of MBean(s) expected to be referenced in corresponding role
	String refMBeanClassName = null;

    // Description of role
	String description = null;

    // Read access mode: true if role is readable
	boolean isReadable = true;

    // Write access mode: true if role is writable
	boolean isWritable = true;

    // Minimum degree (i.e. minimum number of referenced MBeans in
    // corresponding role)
	int minDegree = 1;

    // Maximum degree (i.e. maximum number of referenced MBeans in
    // corresponding role)
	int maxDegree = 1;


	/**
	 * Copy constructor
	 *
	 * @exception java.lang.IllegalArgumentException - if null parameter
	 *
	 * @param theRoleInfo RoleInfo object
	 */
	public RoleInfo(RoleInfo theRoleInfo) throws IllegalArgumentException
	{
		if(theRoleInfo == null)
			throw new IllegalArgumentException("Cannot create RoleInfo for null values");

		if(!(theRoleInfo instanceof RoleInfo) )
			throw new IllegalArgumentException();

		this.roleName = theRoleInfo.getName();
		this.refMBeanClassName = theRoleInfo.getRefMBeanClassName();
		this.description = theRoleInfo.getDescription();
		this.isReadable = theRoleInfo.isReadable();
		this.isWritable = theRoleInfo.isWritable();
		this.minDegree = theRoleInfo.getMinDegree();
		this.maxDegree = theRoleInfo.getMaxDegree();
	}

	/**
	 * Constructor to create RoleInfo
	 *
	 * @param  theName - name of the role
	 *
	 * @param  theRefMBeanClassName - name of the class of MBean(s) expected
	 * 				to be referenced in corresponding role Readable and
	 * 				IsWritable defaulted to true.
	 *         		Minimum and maximum degrees defaulted to 1.
	 * 				Description of role defaulted to null.
	 *
	 * @exception java.lang.IllegalArgumentException - if null parameter
	 *
	 * @exception java.lang.ClassNotFoundException - if the class
	 * 				theMBeanClassName does not exist.
	 *
	 * @exception NotCompliantMBeanException - if the class
	 * 				theRefMBeanClassName is not a MBean class.
	 */
	public RoleInfo(String theName, String theRefMBeanClassName)
         				throws IllegalArgumentException,
                				ClassNotFoundException,
		        				NotCompliantMBeanException
	{
		if( theName == null || theRefMBeanClassName == null)
			throw new IllegalArgumentException("Cannot create a RoleInfo with null values ");

		this.roleName = theName;
		this.refMBeanClassName = theRefMBeanClassName;

		this.isReadable = true;
		this.isWritable = true;

		this.minDegree = 1;
		this.maxDegree = 1;

		this.description = null;
		Class cl = Class.forName(theRefMBeanClassName);
		testForCompliance(theRefMBeanClassName);

		try
		{
			Object obj = cl.newInstance();
			DefaultDynamicMBean mbean = new DefaultDynamicMBean(obj);
			if(obj == null)
			{
				throw new NotCompliantMBeanException();
			}
		}
		catch(Exception e)
		{
			if(e instanceof NotCompliantMBeanException)
			{
				throw (NotCompliantMBeanException)e;
			}
			//e.printStackTrace();
		}
	}

	/**
	 * Constructor to create RoleInfo
	 *
	 * @param theName - name of the role
	 *
	 * @param theRefMBeanClassName - name of the class of MBean(s) expected to
	 * 				be referenced in corresponding role
	 *
	 * @param theIsReadable - flag to indicate if the corresponding role can be read
	 *
	 * @param theIsWritable - flag to indicate if the corresponding role can be set
	 * 				Minimum and maximum degrees defaulted to 1.
	 * 				Description of role defaulted to null.
	 *
	 * @exception java.lang.IllegalArgumentException - if null parameter
	 *
	 * @exception java.lang.ClassNotFoundException - if the class
	 * 				theMBeanClassName does not exist.
	 *
	 * @exception NotCompliantMBeanException - if the class
	 * 				theRefMBeanClassName is not a MBean class.
	 */
	public RoleInfo(String theName, String theRefMBeanClassName,
                	boolean theIsReadable, boolean theIsWritable)
         	throws  IllegalArgumentException,
                	ClassNotFoundException,
					NotCompliantMBeanException
	{
		if( theName ==null || theRefMBeanClassName == null)
			throw new IllegalArgumentException("Cannot create a RoleInfo with null values ");

		this.roleName = theName;
		this.refMBeanClassName = theRefMBeanClassName;

		this.isReadable = theIsReadable;
		this.isWritable = theIsWritable;

		this.minDegree = 1;
		this.maxDegree = 1;

		this.description = null;
		Class cl = Class.forName(theRefMBeanClassName);
		testForCompliance(theRefMBeanClassName);

		try
		{
			Object obj = cl.newInstance();
			DefaultDynamicMBean mbean = new DefaultDynamicMBean(obj);
			if(obj == null)
			{
				throw new NotCompliantMBeanException();
			}
		}
		catch(Exception e)
		{
			if(e instanceof NotCompliantMBeanException)
			{
				throw (NotCompliantMBeanException)e;
			}
			//e.printStackTrace();
		}

	}

	/**
	 * Constructor to create RoleInfo
	 *
	 * @param theName - name of the role
	 *
	 * @param theRefMBeanClassName - name of the class of MBean(s) expected to
	 * 				be referenced in corresponding role
	 *
	 * @param theIsReadable - flag to indicate if the corresponding role can be read
	 *
	 * @param theIsWritable - flag to indicate if the corresponding role can be set
	 *
	 * @param theMinDegree - minimum degree for role, i.e. minimum number of
	 * 				MBeans to provide in corresponding role Must be less or
	 * 				equal than theMaxDegree.
	 *           	(ROLE_CARDINALITY_INFINITY for unlimited)
	 *
	 * @param theMaxDegree - maximum degree for role, i.e. maximum number of
	 * 				MBeans to provide in corresponding role Must be greater or
	 * 				equal than theMinDegree
	 *           	(ROLE_CARDINALITY_INFINITY for unlimited)
	 *
	 * @param theDescription - description of the role (can be null)
	 *
	 * @throws java.lang.IllegalArgumentException - if null parameter
	 *
	 * @throws InvalidRoleInfoException - if the minimum degree is greater than
	 * 				the maximum degree.
	 *
	 * @throws java.lang.ClassNotFoundException - if the class
	 * 				theRefMBeanClassName does not exist.
	 *
	 * @throws NotCompliantMBeanException - if the class
	 * 				theRefMBeanClassName is not a MBean class.
	 */
	public RoleInfo(String theName,
			        String theRefMBeanClassName,
				    boolean theIsReadable,
					boolean theIsWritable,
	                int theMinDegree,
		            int theMaxDegree,
				    String theDescription)
		throws 		IllegalArgumentException,
	                InvalidRoleInfoException,
    	            ClassNotFoundException,
        	        NotCompliantMBeanException
	{
		if( theName ==null || theRefMBeanClassName == null)
			throw new IllegalArgumentException("Cannot create a RoleInfo with null values ");

		if(theMaxDegree != -1 &&( (theMinDegree == -1) || (theMinDegree > theMaxDegree)))
		{
			throw new InvalidRoleInfoException("The MinDegree is greater than MaxDegree");
		}

		if(theMaxDegree < -1 || theMinDegree < -1)
			throw new InvalidRoleInfoException("The Degrees specified for the RoleInfo are not valid !!!!");

		this.roleName = theName;
		this.refMBeanClassName = theRefMBeanClassName;

		this.isReadable = theIsReadable;
		this.isWritable = theIsWritable;

		this.minDegree = theMinDegree;
		this.maxDegree = theMaxDegree;

		this.description = theDescription;
		Class cl = Class.forName(theRefMBeanClassName);
		testForCompliance(theRefMBeanClassName);

		try
		{
			Object obj = cl.newInstance();
			DefaultDynamicMBean mbean = new DefaultDynamicMBean(obj);
			if(obj == null)
			{
				throw new NotCompliantMBeanException();
			}
		}
		catch(Exception e)
		{
			if(e instanceof NotCompliantMBeanException)
			{
				throw (NotCompliantMBeanException)e;
			}
			//e.printStackTrace();
		}
	}

	/**
	 * Returns a boolean to specify if given value is less or equal than
	 * expected maximum degree (true if yes)
	 *
	 * @param theValue - value
	 *
	 * @return true if less or equal than maximum degree, false else
	 */
	public boolean checkMaxDegree(int theValue)
	{
		/*
		if(this.maxDegree != -1 && theValue <= this.maxDegree)
			return true;
		else
			return false;
		*/

        return theValue >= -1 && (maxDegree == -1 || theValue != -1 && theValue <= maxDegree);
	}

	/**
	 * Returns a boolean to specify if given value is greater or equal than
	 * expected minimum degree (true if yes)
	 *
	 * @param theValue - value
	 *
	 * @return true if greater or equal than minimum degree, false else
	 */
	public boolean checkMinDegree(int theValue)
	{
		/*
		if(this.minDegree != -1 &&	theValue >= this.minDegree)
			return true;
		else
			return false;
		*/

		return theValue >= -1 && (minDegree == -1 || theValue >= minDegree);
	}

	/**
	 * Returns description text for the role
	 *
	 * @return Description text for the role
	 */
	public String getDescription()
	{
		return this.description;
	}

	/**
	 * Returns maximum degree for corresponding role reference
	 *
	 * @return Maximum degree for corresponding role reference
	 */
	public int getMaxDegree()
	{
		return this.maxDegree;
	}

	/**
	 * Returns minimum degree for corresponding role reference
	 *
	 * @return Minimum degree for corresponding role reference
	 */
	public int getMinDegree()
	{
		return this.minDegree;
	}

	/**
	 * Returns the name of the role
	 *
	 * @return The name of the role
	 */
	public String getName()
	{
		return this.roleName;
	}

	/**
	 * Returns name of class of MBean expected to be referenced in corresponding role
	 *
	 * @return The string representation of name of class of MBean expected to
	 * 				be referenced in corresponding role
	 */
	public String getRefMBeanClassName()
	{
		return this.refMBeanClassName;
	}

	/**
	 * Returns read access mode for the role (true if it is readable)
	 *
	 * @return True if it is readable otherwise false
	 */
	public boolean isReadable()
	{
		return this.isReadable;
	}

	/**
	 * Returns write access mode for the role (true if it is writable)
	 *
	 * @return True if it is writable otherwise false
	 */
	public boolean isWritable()
	{
		return this.isWritable;
	}

	/**
	 * Prints a string describing the role info
	 *
	 * Overrides:
	 *           toString in class java.lang.Object
	 *
	 * @return String representation of the role info
	 */
	public String toString()
	{
		StringBuffer str = new StringBuffer();

		str.append( "RoleInfo Name:"+getName()+"; isReadable:"+isReadable()+"; isWritable:"+isWritable()+
		            "; Description:"+getDescription()+"; Minimum Degree:"+getMinDegree()+
		            "; Maximum Degree:"+getMaxDegree()+"; MBean Class:"+getRefMBeanClassName() );

		return str.toString();
	}

	//--------------------------- Private methods --------------------------//

  	private void testForCompliance(String className) throws NotCompliantMBeanException
	{
		Class cl = null;

		try
		{
	  		cl = Class.forName(className);
	  		int i = cl.getModifiers();

			  if(Modifier.isAbstract(i))
	              throw new NotCompliantMBeanException("The MBean cannot be a abstract class");

			Constructor constructors[] = cl.getConstructors();

			if(constructors.length == 0)
	            throw new NotCompliantMBeanException("The MBean does not have a public constructor");
	  	}
		catch(ClassNotFoundException ee)
	  	{
			//ee.printStackTrace();
			//log.warn("ClassNotFoundException",ee);
  		}

		Class superClass = null;
  		superClass = cl;
  		Class class1 = null;
  		Class class2 = null;
  		boolean flag = false;

  		for(;superClass != null;superClass = superClass.getSuperclass())
		{
  			Class[] interfaces = superClass.getInterfaces();
  			boolean checkFlag = false;
  			for(int j=0;j<interfaces.length;j++)
			  {
			  	Class tempClass = checkForMBeanInterface(interfaces[j],superClass.getName());
  				if(tempClass != null)
				  {
  					flag = true;
  					class1 = tempClass;
  					class2 = superClass;
  				}
  				if(checkForDynamicMBeanInterface(interfaces[j]))
				  {
  					flag = true;
  					checkFlag = true;
  				}
  				if(class1 != null && superClass.equals(class2) && checkFlag)
                      throw new NotCompliantMBeanException(className + " implements javax.management.DynamicMBean and MBean interface");
  			}
  			if(flag)
  				break;
  		}
          if(!flag)
              throw new NotCompliantMBeanException(className + " does not implement the " + className+"MBean" + " interface or the DynamicMBean interface");
  	}

  	private Class checkForMBeanInterface(Class class1, String s)
	{
		if(class1.getName().compareTo(s + "MBean") == 0)
			return class1;

		Class aclass[] = class1.getInterfaces();

		for(int i = 0; i < aclass.length; i++)
		{
			if(aclass[i].getName().compareTo(s + "MBean") == 0)
				return aclass[i];
		}

		return null;
	}

  	private boolean checkForDynamicMBeanInterface(Class class1)
	{
		if(class1.getName().compareTo("javax.management.DynamicMBean") == 0)
			return true;

		Class aclass[] = class1.getInterfaces();

		for(int i = 0; i < aclass.length; i++)
			if(aclass[i].getName().compareTo("javax.management.DynamicMBean") == 0)
				return true;

		return false;
	}

 	private boolean isValidMBeanInfo(MBeanInfo mbInfo)
 	{
		if(mbInfo.getClassName() == null)
		{
			return false;
		}

		MBeanConstructorInfo[] consInfo = mbInfo.getConstructors();
		if(consInfo == null)
		{
			return false;
		}
		else if(consInfo.length == 0)
		{
			return false;
		}

		return true;
 	}
}