/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.relation;

import java.io.Serializable;
import java.util.List;
import java.util.ArrayList;

import javax.management.ObjectName;

/**
 * Represents an unresolved role: a role not retrieved from a relation due to
 * a problem. It provides the role name, value (if problem when trying to set
 * the role) and an integer defining the problem (constants defined in RoleStatus).
 */
public class RoleUnresolved implements Serializable
{
	// role name
	String roleName = null;

	// list of role values (contains list of ObjectNames)
	List roleValueList = null;

	// problem type
	int pbType;

	/**
	 * Constructor to create RoleUnresolved
	 *
	 * @param theRoleName - name of the role
	 *
	 * @param theRoleValue - value of the role (if problem when setting the role)
	 *
	 * @param thePbType - type of problem (according to known problem types,
	 * 				listed as static final members).
	 *
	 * @exception java.lang.IllegalArgumentException - if null parameter
	 * 				or incorrect problem type
	 */
	public RoleUnresolved(String theRoleName, List theRoleValue, int thePbType)
								throws IllegalArgumentException
	{
		if(thePbType < 0 || thePbType > 7)
			throw new IllegalArgumentException();

		if(theRoleName == null )
			throw new IllegalArgumentException();

		this.roleName = theRoleName;
		this.roleValueList = theRoleValue;
		this.pbType = thePbType;
	}

	/**
	 * Clone. To create a duplicate copy of the current object (i.e) RoleUnresolved
	 *
	 *  Overrides:
	 *           clone in class java.lang.Object
	 * @return
	 *           an independent clone
	 */
	public Object clone()
	{
		return new RoleUnresolved(getRoleName(), getRoleValue(), getProblemType());
	}

	/**
	 * Retrieves problem type
	 *
	 * @return an integer corresponding to a problem, those being described
	 * 				as static final members of current class.
	 */
	public int getProblemType()
	{
		return this.pbType;
	}

	/**
	 * Retrieves role name
	 *
	 * @return Name of the role
	 */
	public String getRoleName()
	{
		return this.roleName;
	}

	/**
	 * Retrieves role value
	 *
	 * @return an ArrayList of ObjectName objects, the one provided to be set
	 * 				in given role. Null if the unresolved role is returned
	 * 				for a read access.
	 */
	public List getRoleValue()
	{
		return this.roleValueList;
	}

	/**
	 * Sets problem type
	 *
	 * @param thePbType - integer corresponding to a problem. Must be one of
	 * 				those described as static final members of current class.
	 *
	 * @exception java.lang.IllegalArgumentException - if incorrect problem type
	 */
	public void setProblemType(int thePbType)
						throws IllegalArgumentException
	{
		if(thePbType < 1 || thePbType > 7)
			throw new IllegalArgumentException();
	}

	/**
	 * Sets role name
	 *
	 * @exception java.lang.IllegalArgumentException - if null parameter
	 */
	public void setRoleName(String theRoleName)
					throws IllegalArgumentException
	{
		if(theRoleName == null)
			throw new IllegalArgumentException();

		this.roleName = theRoleName;
	}

	/**
	 * Sets role value
	 *
	 * @param theRoleValue - ArrayList of ObjectName objects for referenced
	 * 				MBeans not set in role.
	 */
	public void setRoleValue(List theRoleValue)
	{
		this.roleValueList = (ArrayList)theRoleValue;
	}

	/**
	 * Prints a string describing the role
	 *
	 * Overrides:
	 *       toString in class java.lang.Object
	 *
	 * @return String representation of the role
	 */
	public String toString()
	{
        StringBuffer str = new StringBuffer();
        str.append(getRoleName());
        str.append(":");

        for(int i=0;i<roleValueList.size();i++)
		{
            str.append( ( (ObjectName)(roleValueList.get(i)) ).toString() );
            str.append(",");
        }

		String toRet = str.toString();

        return toRet.substring(0,toRet.length() - 1);
	}
}