/**
* The XMOJO Project 5
* Copyright  2003 XMOJO.org. All rights reserved.

* NO WARRANTY

* BECAUSE THE LIBRARY IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY FOR
* THE LIBRARY, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
* OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
* PROVIDE THE LIBRARY "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
* OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE ENTIRE RISK AS
* TO THE QUALITY AND PERFORMANCE OF THE LIBRARY IS WITH YOU. SHOULD THE
* LIBRARY PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
* REPAIR OR CORRECTION.

* IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL
* ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR REDISTRIBUTE
* THE LIBRARY AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
* GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OR INABILITY TO USE THE LIBRARY (INCLUDING BUT NOT LIMITED TO LOSS OF
* DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
* PARTIES OR A FAILURE OF THE LIBRARY TO OPERATE WITH ANY OTHER SOFTWARE),
* EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGES.
**/

package javax.management.timer;

import java.io.Serializable;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.management.ObjectName;
import javax.management.MBeanRegistration;
import javax.management.MBeanServer;
import javax.management.NotificationBroadcasterSupport;
import javax.management.InstanceNotFoundException;

//import com.adventnet.agent.utilities.scheduler.Scheduler;

import com.adventnet.agent.logging.Log;
import com.adventnet.agent.logging.LogFactory;

/**
 * This class provides the implementation of the timer MBean. The timer MBean
 * sends out an alarm at a specified time that wakes up all the listeners
 * registered to receive timer notifications.
 * <P>
 * This class manages a list of dated timer notifications. A method allows
 * users to add/remove as many notifications as required. When a timer
 * notification is emitted by the timer and becomes obsolete, it is
 * automatically removed from the list of timer notifications.
 * <BR>Additional timer notifications can be added into regularly repeating
 * notifications.
 * <P>
 *
 * Note:
 * <OL>
 * <LI>All notifications before the time when the <CODE>addNotification</CODE>
 * method is called are ignored, irrespective of the
 * <CODE>sendPastNotifications</CODE> flag.
 * <LI>When sending timer notifications, the timer updates the notification
 * sequence number irrespective of the notification type.
 * <LI>The timer service relies on the system date of the host where the
 * <CODE>Timer</CODE> class is loaded. Callers may be unable to add
 * notifications to the timer, and listeners may recieve untimely notifications
 * if their host has a different system date. To avoid such problems,
 * synchronize the system date of all host machines where timing is needed.
 * </OL>
 */
public class Timer extends NotificationBroadcasterSupport
						implements TimerMBean,MBeanRegistration, Serializable
{
	/**
	 * Number of milliseconds in one second.
	 * This constant can be used in addNotification method.
	 */
    public static final long ONE_SECOND = 1000;

    /**
	 * Number of milliseconds in one minute.
	 * This constant can be used in addNotification method.
	 */
    public static final long ONE_MINUTE =	60 * ONE_SECOND;

	/**
	 * Number of milliseconds in one hour.
	 * This constant can be used in addNotification method.
	 */
    public static final long ONE_HOUR	  = 60 * ONE_MINUTE;

	/**
	 * Number of milliseconds in one day.
	 * This constant can be used in addNotification method.
	 */
    public static final long ONE_DAY    =	24 * ONE_HOUR;

	/**
	 * Number of milliseconds in one week.
	 * This constant can be used in addNotification method.
	 */
    public static final long ONE_WEEK	  =	7  * ONE_DAY;

	/**
	 * Scheduler that will be used to schedule the TimerNotifications.
	 */
    private Scheduler scheduler           =     null;

  	/**
	 * Variable representing the unique Notification ID.
	 */
    private int notificationID	          =     0;

    /**
	 * Hashtable where the Timer Tasks are stored.
	 */
    private Hashtable timerTable           =     new Hashtable();

    /**
	 * Boolean to indicate if pastNotifications flag is enabled or not.
     * This attribute is used to specify if the timer has to send past
	 * notifications after start.
     * <BR>The default value is set to <CODE>false</CODE>.
	 */
    private boolean sendPastNotifications =     false;

    /**
	 * Variable to store the ObjectName of the Timer.
	 */
    private ObjectName objectName         =     null;

    /**
	 * Variable indicating the state of the Timer.
	 */
    private boolean active                =     false;

    /**
	 * The Logger instance used for Logging purposes.
	 */
    private Log log = null;

	// the following are present here to make serialization compatible with RI

    /**
     * The notification counter ID.
     * Used to keep the max key value inserted into the timer table.
     */
    private int counterID = 0;

	// not used
    private String dbgTag = "";

	// end of dummy variables for serialization compatible

	/**
	 * Default Constructor
	 */
    public Timer()
    {
    	try
		{
			log=LogFactory.getInstance("JMX");
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}

		log.trace("Timer Instantiated");
    }

    // Mbean Registration methods

    /**
     * Allows the timer MBean to perform any operations it needs before
	 * being registered in the MBean server.
     *
     * @param server The MBean server in which the timer MBean will be registered.
     *
     * @param name The object name of the timer MBean.
     *
     * @return The name of the timer MBean registered.
     *
     * @exception java.lang.Exception
     */
    public ObjectName preRegister(MBeanServer server,ObjectName objectName)
	                             					throws Exception
	{
		this.objectName = ( (objectName == null)
									? new ObjectName("Services","type","Timer")
									: objectName );
		if(scheduler == null)
			scheduler = Scheduler.createScheduler(this.objectName.toString(),2);

		return this.objectName;
    }

    /**
     * Allows the timer MBean to perform any operations needed after having been
     * registered in the MBean server or after the registration has failed.
     * <P>
     * Not used in this context.
     */
    public void postRegister(Boolean registrationDone)
    {
    }

    /**
     * Allows the timer MBean to perform any operations it needs before
	 * being de-registered by the MBean server.
     * <P>
     * Stops the timer.
     *
     * @exception java.lang.Exception
     */
    public void preDeregister()throws java.lang.Exception
    {
		stop();
    }

    /**
     * Allows the timer MBean to perform any operations needed after having been
     * de-registered by the MBean server.
     */
    public void postDeregister()
    {
		if(scheduler != null)
		{
			scheduler.deregisterThisScheduler("TIMER");
		}
    }

    /**
	 * This method starts the timer.
     * <P>
     * If there is one or more timer notifications before the time in the
	 * list of notifications, the notification is sent according to the
	 * <CODE>sendPastNotifications</CODE> flag and then, updated according
	 * to its period and remaining number of occurences. If the timer
	 * notification date remains earlier than the current date, this
	 * notification is just removed from the list of notifications.
     */
	public synchronized void start()
	{
		try
		{
			if(objectName == null)
				objectName = new ObjectName("Services","type","Timer");
		}
		catch(Exception e){}

		if(scheduler == null)
		{
			scheduler = Scheduler.createScheduler(this.objectName.toString(),2);
			Enumeration e = timerTable.keys();

			while(e.hasMoreElements())
			{
				Task t = (Task)(timerTable.get(e.nextElement()));
				scheduler.scheduleTask(t,
							new Date(t.getTimerNotification().getTimeStamp()));
			}
		}

	    sendExpiredNotifications(sendPastNotifications);

	    if(active == false || ! scheduler.isAlive())
	    {
	    	scheduler.start();

			//printing the values ...
			Enumeration e = timerTable.keys();
			while(e.hasMoreElements())
			{
				Task t = (Task)(timerTable.get(e.nextElement()));
			}

			active=true;
			return;
		}
	}

	/**
	 * This method stops the timer.
	 */
	public synchronized void stop()
	{
		scheduler.killScheduler();
	    scheduler.cleanUp();
	    scheduler = null;

	    active=false;
	}

	/**
     * This method creates a new timer notification with the specified
	 * <CODE>type</CODE>, <CODE>message</CODE> and <CODE>userData</CODE>
	 * and inserts it into the list of notifications with a given date and
	 * a null period and number of occurences.
     * <P>
     * The timer notification will be handled once at the specified date.
     *
     * @param type The timer notification type.
     *
     * @param message The timer notification detailed message.
     *
     * @param userData The timer notification user data object.
     *
     * @param date The date when the notification occurs.
     *
	 * @return An instance of java.lang.Integer which is the identifier of
	 * 				the new created timer notification.
     *
     * @exception java.lang.IllegalArgumentException The date notification
	 * 				is before the current date.
	 */
	public Integer addNotification(String type,
								   String message,
								   Object userData,
								   Date date)
							throws IllegalArgumentException
	{
	    log.trace("addNotificaation Called with 0 period and 0 occurences");
		return addNotification(type, message, userData, date, 0, 1);
	}

	/**
     * This method creates a new timer notification with the specified
	 * <CODE>type</CODE>, <CODE>message</CODE> and <CODE>userData</CODE>
	 * and inserts it into the list of notifications with a given date
     * and period and a null number of occurences.
     * <P>
     * The timer notification will repeat continuously using the timer period.
     *
     * @param type The timer notification type.
     *
     * @param message The timer notification detailed message.
     *
     * @param userData The timer notification user data object.
     *
     * @param date The date when the notification occurs.
     *
     * @param period The period of the timer notification (in milliseconds).
     *
	 * @return  An instance of java.lang.Integer which is the identifier of
	 * 				the new created timer notification.
	 *
	 * @exception java.lang.IllegalArgumentException - The period is negative
	 * 				or the date notification is before the current date.
	 */
	public Integer addNotification(String type,
								   String message,
								   Object userData,
								   Date date,
								   long period)
							throws IllegalArgumentException
	{
	    log.trace("addNotification called with 0 occurences");
		return addNotification(type, message, userData, date, period, 0);
	}

    /**
     * This method creates a new timer notification with the specified
	 * <CODE>type</CODE>, <CODE>message</CODE> and <CODE>userData</CODE> and
	 * inserts it into the list of notifications with a given date, period
	 * and number of occurences.
     * <P>
     * If the timer notification to be inserted has a date that is before
	 * the current date, the method attempts to update the notification
	 * using the defined period. If the defined period is null, the method
	 * generates an exception.
     * <P>
     * If the timer notification has a non null period, the date of the
	 * notification is updated by adding the period until the notification
	 * date is later than the current date. The timer notification is then
	 * inserted into the list of notifications.
     * <P>
     * If the specified number of occurences is non null, the previous
	 * mechanism is employed. The number of times that the period is added
	 * is limited by the specified number of occurences. If the notification
	 * date remains earlier than the current date, an exception is generated.
     * <P>
     * Note that once the timer notification has been added into the list of
	 * notifications, its associated date, period and number of occurences
	 * cannot be updated.
     *
     * @param type The timer notification type.
     *
     * @param message The timer notification detailed message.
     *
     * @param userData The timer notification user data object.
     *
     * @param date The date when the notification occurs.
     *
     * @param period The period of the timer notification (in milliseconds).
     *
     * @param nbOccurences The total number the timer notification will be emitted.
     *
     * @return The identifier of the new created timer notification.
     *
     * @exception java.lang.IllegalArgumentException The period or the number
	 * 				of occurences is negative or the date notification is
	 * 				before the current date.
     */
	public Integer addNotification(String type, String message, Object userData,
								   Date date, long period, long nbOccurences)
						throws IllegalArgumentException
	{
	    long nbOccurences_local = nbOccurences;

	 	if(date == null)
	 	{
			throw new IllegalArgumentException("Timer notification date cannot be null!!!");
		}

	    // Negative periods and nbOccurences Not allowed
	    if(period<0)
	    {
			throw new IllegalArgumentException("Period cannot be Negative");
	    }

	    if(nbOccurences<0)
	    {
			throw new IllegalArgumentException("Number of Occurences cannot be Zero");
	    }

	    if( (date.before(new Date())) && (period==0))
	    {
			throw new IllegalArgumentException("Date specified is earlier than the current Date");

	    }
	    else if((date.before(new Date()))&&(period>0 )&&(nbOccurences>0))
	    {
			while(nbOccurences_local>0)
			{
			    date = new Date(date.getTime()+period);
				if(date.after(new Date()))
				{
					break;
				}
			    --nbOccurences_local;
			}
			if(date.before(new Date()))
				throw new IllegalArgumentException("Date specified is earlier than the current Date");
		}
	    else if((date.before(new Date()))&&(period>0))
	    {
			log.trace("Date before currentDate and period >  0");
			while(true)
			{
		    	date=new Date(new Date().getTime()+period);
		    	if(date.after(new Date()))
		    		break;
			}
	    }

		try
		{
			if(objectName == null)
				objectName = new ObjectName("Services","type","Timer");
		}
		catch(Exception e){}

		if(scheduler == null)
			scheduler = Scheduler.createScheduler(this.objectName.toString(),2);

	    //create TimerNotification and Schedule a task to send that
	    Integer id = createNotificationID();

	    TimerNotification timerNotification = new TimerNotification(id,type,
	    								objectName,message,userData,date);

	    Task task=new Task(timerNotification,period,nbOccurences);

	    timerTable.put(id,task);
	    scheduler.scheduleTask(task,date);

	    return id;
   	}

	/**
	 * This method gets all timer notification identifiers registered into
	 * the list of notifications.
	 *
	 * @return A vector of Integer objects containing all the timer
	 * 				notification identifiers. The vector is empty if there
	 * 				is no timer notification registered for this timer MBean.
	 */
	public Vector getAllNotificationIDs()
	{
	    //return the Notification IDs
	    Vector vector = new Vector();
		for(Enumeration e=timerTable.keys();e.hasMoreElements();)
	    {
			vector.add(e.nextElement());
	    }
	    return vector;
	}

	/**
	 * This method gets a copy of the date associated to a timer notification.
	 *
	 * @param id - The timer notification identifier.
	 *
	 * @return A copy of the date or null if the identifier is not mapped to
	 * 				any timer notification registered for this timer MBean.
	 */
	public Date getDate(Integer id)
	{
	    Task task=(Task)timerTable.get(id);

	    if(task != null)
		    return new Date(task.getTimerNotification().getTimeStamp());
		else
			return null;
	}

	/**
	 *	This method gets the number of timer notifications registered into the
	 *	list of notifications.
	 *
	 *	@return The number of timer notifications.
	 */
	public int getNbNotifications()
	{
	    return timerTable.size();
	}

	/**
	 * This method gets a copy of the remaining number of occurences
	 * associated to a timer notification.
	 *
	 * @param id - The timer notification identifier.
	 *
	 * @return A copy of the remaining number of occurences or null if
	 * 				the identifier is not mapped to any timer notification
	 * 				registered for this timer MBean.
	 */
	public Long getNbOccurences(Integer id)
	{
	    return new Long(((Task)timerTable.get(id)).getNbOccurences());
	}

	/**
	 * This method gets all the identifiers of timer notifications
	 * corresponding to the specified type.
	 *
	 * @param type - The timer notification type.
	 *
     * @return A vector of <CODE>Integer</CODE> objects containing all the
	 * 				identifiers of timer notifications with the specified
	 * 				<CODE>type</CODE>. <BR>The vector is empty if there is no
	 * 				timer notifications registered for this timer MBean
	 * 				with the specified <CODE>type</CODE>.
	 */
	public Vector getNotificationIDs(String type)
	{
	    Vector vector= new Vector();

	    for(Enumeration e=timerTable.elements();e.hasMoreElements();)
	    {
			TimerNotification timerNotification=((Task)e.nextElement()).getTimerNotification();
			if(timerNotification.getType().equals(type))
			{
			    vector.add(timerNotification.getNotificationID());
			}
	    }
	    return vector;
	}

	/**
	 * This method gets the timer notification detailed message corresponding
	 * to the specified identifier.
	 *
	 * @param id - The timer notification identifier.
	 *
	 * @return The timer notification detailed message or null if the
	 * 				identifier is not mapped to any timer notification
	 * 				registered for this timer MBean.
	 */
	public String getNotificationMessage(Integer id )
	{
	    return ((Task)timerTable.get(id)).getTimerNotification().getMessage();
	}

	/**
	 * This method gets the timer notification type corresponding to the
	 * specified identifier.
	 *
	 * @param id - The timer notification identifier.
	 *
	 * @return The timer notification type or null if the identifier is not
	 * 				mapped to any timer notification registered for this timer MBean.
	 */
	public String getNotificationType(Integer id )
	{
	    return ((Task)timerTable.get(id)).getTimerNotification().getType();
	}

	/**
	 * This method gets the timer notification user data object corresponding
	 * to the specified identifier.
	 *
	 * @param id - The timer notification identifier.
	 *
	 * @return The timer notification user data object or null if the
	 * 				identifier is not mapped to any timer notification
	 * 				registered for this timer MBean.
	 */
	public Object getNotificationUserData(Integer id )
	{
	    return ((Task)timerTable.get(id)).getTimerNotification().getUserData();
	}

	/**
	 * This method gets a copy of the period (in milliseconds) associated
	 * to a timer notification.
	 *
	 * @param id - The timer notification identifier.
	 *
	 * @return A copy of the period or null if the identifier is not mapped
	 * 				to any timer notification registered for this timer MBean.
	 */
	public Long getPeriod(Integer id )
	{
	    return new Long(((Task)timerTable.get(id)).getPeriod());
	}

	/**
	 * This method gets the flag indicating whether or not the timer
	 * sends past notifications.
	 *
	 * @return The past notifications sending on/off flag value.
	 */
	public boolean getSendPastNotifications()
	{
	    return sendPastNotifications;
	}

	/**
	 * This method tests whether the timer MBean is active. A timer MBean is
	 * marked active when the start method is called. It becomes inactive
	 * when the stop method is called.
	 *
	 * @return true if the timer MBean is active, false otherwise.
	 */
	public boolean isActive()
	{
	    return active;
	}

	/**
	 * This method tests whether the list of timer notifications is empty.
	 *
	 * @return true if the list of timer notifications is empty, false otherwise.
	 */
	public boolean isEmpty()
	{
	    return timerTable.isEmpty();
	}

	/**
	 *	This method removes all the timer notifications from the list of
	 *	notifications and resets the counter used to update the timer
	 *	notification identifiers.
	 */
	public void removeAllNotifications()
	{
	    for(Enumeration e=timerTable.keys();e.hasMoreElements();)
	    {
			Task task = (Task)(timerTable.get(e.nextElement()));
		    scheduler.removeTask(task);
		    timerTable.remove(task.getTimerNotification().getNotificationID());
	    }

	    timerTable.clear();
	}

	/**
	 * This method Removes the timer notification corresponding to the
	 * specified identifier from the list of notifications.
	 *
	 * @param id - The timer notification identifier.
	 *
	 * @exception InstanceNotFoundException - The specified identifier does
	 * 				not correspond to any timer notification in the list
	 * 				of notifications of this timer MBean.
	 */
	public void removeNotification(Integer id) throws InstanceNotFoundException
	{
	    Task task=(Task)timerTable.get(id);

	    if(task == null)
	    	throw new InstanceNotFoundException("No such Notification in the list of Timer");

	    scheduler.removeTask(task);
	    timerTable.remove(id);
	}

	/**
	 * Removes all the timer notifications corresponding to the specified type
	 * from the list of notifications.
	 *
	 * @param type - The timer notification type.
	 *
	 * @exception InstanceNotFoundException - The specified type does not
	 * 				correspond to any timer notification in the list of
	 * 				notifications of this timer MBean.
	 */
	public void removeNotifications(String type)
		throws InstanceNotFoundException
	{
		boolean temp = false;
	    for(Enumeration e=timerTable.keys();e.hasMoreElements();)
	    {
			Task task=(Task)(timerTable.get(e.nextElement()));
			if(task.getTimerNotification().getType().equals(type))
			{
				temp = true;
			    scheduler.removeTask(task);
			    timerTable.remove(task.getTimerNotification().getNotificationID());
			}
	    }

	    if(! temp)
	    	throw new InstanceNotFoundException("No such Notification in the list of Timer");
	}

	/**
	 * This method Sets the flag indicating whether the timer sends past
	 * notifications or not.
	 *
	 * @param value - The past notifications sending on/off flag value.
	 */
	public void setSendPastNotifications(boolean value)
	{
	    sendPastNotifications=value;
	}


	////////////////////// Inner class for the Timer //////////////////////////

	class Task implements Runnable
	{
		private TimerNotification timerNotification=null;
		private long period=0;
		private long nbOccurences=0;
		private long sequenceNumber=0;

		public Task(TimerNotification timerNotification,long period,
					long nbOccurences)
		{
			//new Throwable().printStackTrace();
			this.timerNotification=timerNotification;
			this.period=period;
			this.nbOccurences=nbOccurences;

			if(period == 0 && nbOccurences == 1)
				fromAddNotification = true;
		}

		public void run()
		{
			scheduler.removeTask(this);
			//timerTable.remove(timerNotification.getNotificationID());
			if((period>0)&& (nbOccurences==0))
			{
				//while(true)
				//{
					setTimerNotification();
					sendNotification(timerNotification);
				//	try{
				//		Thread.sleep(period);
				//	}catch(InterruptedException e){
				//	}
				//}

				//scheduler.removeTask(this);//iv
				TimerNotification tn = clone(timerNotification);
				this.timerNotification = tn;
				//timerTable.put(timerNotification.getNotificationID(),this);
				scheduler.scheduleTask(this,new Date(new Date().getTime()+period));
				//System.out.println(" Removing the id ................. ");
				//timerTable.remove(timerNotification.getNotificationID());//iv
			}
			else if((period> 0)&&(nbOccurences>0))
			{
				//while(getNbOccurences()>0)
					setTimerNotification();
					sendNotification(timerNotification);
				//	try{
				//		Thread.sleep(period);
				//	}catch(InterruptedException e){
				//	}
					nbOccurences = nbOccurences -1;

				if(nbOccurences>0)
				{
					//construct a new TimerNotification object and schdule the task

					TimerNotification tn = clone(timerNotification);
					this.timerNotification = tn;
					//timerTable.put(timerNotification.getNotificationID(),this);
					scheduler.scheduleTask(this,new Date(new Date().getTime()+period));
				}
				else
				{
					timerTable.remove(timerNotification.getNotificationID());
				}
				/*else if(nbOccurences == 0){
					period = 0;
				}*/

			}
			else if((period == 0)&&(nbOccurences == 0));
			else
			{
				setTimerNotification();
				sendNotification(timerNotification);
				timerTable.remove(timerNotification.getNotificationID());
			}
		}

		public synchronized  TimerNotification getTimerNotification()
		{
			return timerNotification;
		}

		private synchronized void setTimerNotification()
		{
			timerNotification.setSequenceNumber(sequenceNumber++);
			timerNotification.setTimeStamp(new Date().getTime());
		}

		public long getPeriod()
		{
			return period;
		}

		boolean fromAddNotification = false;
		public synchronized long getNbOccurences()
		{
			if(fromAddNotification)
			{
				return 0;
			}
			return nbOccurences;
		}
		public synchronized long getOriginalNbOccurences()
		{
			return nbOccurences;
		}

		public  synchronized void setNbOccurences(long l)
		{
			nbOccurences=l;
		}
		public void setSequenceNumber(long seqNumber)
		{
			this.sequenceNumber = seqNumber;
		}

		public TimerNotification clone(TimerNotification timerNotification)
		{
			TimerNotification tn = new TimerNotification(
									timerNotification.getNotificationID(),
									timerNotification.getType(),
									(ObjectName)timerNotification.getSource(),
									timerNotification.getMessage(),
									timerNotification.getUserData(),
									new Date(timerNotification.getTimeStamp())
								);
			tn.setSequenceNumber(timerNotification.getSequenceNumber());
			tn.setTimeStamp(timerNotification.getTimeStamp());
			return tn;
		}
	}

	//---------------------------- Private method ----------------------------//

	/**
 	 * Creates a Unique Notification Identifier
 	 */
    private Integer createNotificationID()
    {
		return new Integer(++notificationID);
    }

    /**
     *	Sends the past Notifications for the Timer.
     */
	private void sendExpiredNotifications(boolean sendPastNotifications)
	{
		Enumeration e = timerTable.keys();

		for(;e.hasMoreElements();)
		{
			Task t = (Task)(timerTable.get(e.nextElement()));
			synchronized(t)
			{
				if((new Date(t.getTimerNotification().date)).before(new Date()))
				{
					long currTime = System.currentTimeMillis();

					if((currTime > t.getTimerNotification().getTimeStamp()) && t.getOriginalNbOccurences() == 1)
					{
						if(sendPastNotifications)
						{
							t.getTimerNotification().setSequenceNumber(0);
							t.getTimerNotification().setTimeStamp(new Date().getTime());
							sendNotification(t.getTimerNotification());
							t.setSequenceNumber(1);
						}

						timerTable.remove(t.getTimerNotification().getNotificationID());
						scheduler.removeTask(t);
					}
					else if((currTime > t.getTimerNotification().getTimeStamp()) && (t.getPeriod() > 0) &&
							(t.getNbOccurences() == 0))
					{
						//to do : sendPastNotifications ...
					}
					else if((currTime > t.getTimerNotification().getTimeStamp()) && (t.getPeriod() > 0) )
					{
						float pastNotifCount = ( new Long(currTime - t.getTimerNotification().getTimeStamp()).floatValue() )/new Long(t.getPeriod()).floatValue();

						long pastNotifCountInLong = new Float(pastNotifCount).longValue();

						if(pastNotifCount > new Long(pastNotifCountInLong).floatValue())
							pastNotifCountInLong += 1;

						//riskey code: need to change it later, Hyther
						if((t.getNbOccurences() - pastNotifCountInLong) < 1)
							pastNotifCountInLong -= 1;

						t.setNbOccurences(t.getNbOccurences() - pastNotifCountInLong);

                        if(sendPastNotifications)
						{
							long i=0;
							for(;i<pastNotifCountInLong;i++)
							{
								TimerNotification timerNotif = t.clone(t.getTimerNotification());
								timerNotif.setSequenceNumber(i);
								timerNotif.setTimeStamp(new Date().getTime());
								sendNotification(timerNotif);
							}
							t.setSequenceNumber(i);
						}
					}
				}
			}
		}
	}
}//End the Timer class.